#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::RemoteMWS
# state        public
#----------------------------------------------------------------------------------------
# description  base class for MWS remote adapter family
#========================================================================================
package DE_EPAGES::Amazon::API::Object::RemoteMWS;
use base DE_EPAGES::Amazon::API::Object::RemoteAWS;

use DE_EPAGES::Amazon::API::Constants qw ( SP_API_REPORTS_RESSOURCE SP_API_SELLERS_RESSOURCE SP_API_FEEDS_RESSOURCE SP_API_PRODUCTS_PRICING_RESSOURCE );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Net qw( GetHttpContent );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Core::API::Url            qw ( BuildParameterString );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);
use JSON;
use URI::Escape qw(uri_escape_utf8);
use strict;

sub ErrorDefinition {
    return {
        'PARAMETER_MISSING' => 'Missing parameter #Parameter',
        'MissingConstructorArgument' => 'Need to set argument "#Argument" when calling new.',
        'API_CALL_ERROR' => 'API call failed: in sub:#CallName, RC:#ResponseCode, Content:#Content',
    };
}

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = __PACKAGE__->SUPER::new(%args);

    for (qw/MerchantID MarketplaceID/) {
        Error( 'MissingConstructorArgument', { 'Argument' => $_ } ) unless ( $args{$_} or ($args{$_} eq '0') );
    }

    $self->{merchantID}              = $args{'MerchantID'};
    $self->{mwsAuthToken}            = $args{'MWSAuthToken'};
    $self->{marketplaceID}           = $args{'MarketplaceID'};
    $self->{secondaryMarketplaceIDs} = $args{'SecondaryMarketplaceIDs'};

    $self->{SPAPI_EndPoint}         = $args{'SPAPI_EndPoint'};
    $self->{PortalConfig}           = $args{'PortalConfig'};


    # mws uses a 120sec timeout.
    $self->userAgent->timeout(GetDefaultTimeOut); ## no critic qw(:ProhibitMagicNumbers)
    return $self;
}

#SPAPI - getReportList ->  getReports -----------------------------------------------------------------
# see: #https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#get-reports2021-06-30reports
# resource path: GET /reports/2021-06-30/reports
sub getReports {
    my $self = shift;
    my ($hParams) = @_;

    my $hContent = {
        'pageSize'           => 100,
        'processingStatuses' => 'DONE',
    };
    # add params to content
    $hContent->{$_} = $hParams->{$_} foreach keys %$hParams;

    my $URI = $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/reports' ;
    $URI = _buildRequestURL($URI, $hContent);
    my $Response = $self->{PortalConfig}->doRequest('GET', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});

    if ($Response->code == 403) {
        my $hError = from_json($Response->decoded_content);
        my $ahErrors = $hError->{'errors'};

        foreach my $hError (@$ahErrors) {
            if ($hError->{'code'} eq 'Unauthorized') {
                my $PortalConfig = $self->{PortalConfig};
                $PortalConfig->set(
                    {'SPAPI_AccessToken' => undef,
                     'SPAPI_AccessTokenExpiry' => undef,
                     'SPAPI_RefreshToken' => undef,
                     'SPAPI_RefreshTokenExpiry' => undef
                    });

                return { 'reports' => [] };
            }
        }
    }

    my $hReports = [];
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();
    $hReports = from_json($Response->decoded_content);

    # use paging for getting the next reports if we have a nextToken
    while($hReports->{'nextToken'}){
        my $NextTokenURI = $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/reports' .'?nextToken=' . uri_escape_utf8($hReports->{'nextToken'}) ;
        my $NextTokenResponse = $self->{PortalConfig}->doRequest('GET', $NextTokenURI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});
        Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $NextTokenResponse->code, 'Content' => $NextTokenResponse->decoded_content }) unless $NextTokenResponse && $NextTokenResponse->is_success();
        my $hNextTokenReports = from_json($NextTokenResponse->decoded_content);
        push @{$hReports->{'reports'}}, @{$hNextTokenReports->{'reports'}};
        $hReports->{'nextToken'} = $hNextTokenReports->{'nextToken'} ? $hNextTokenReports->{'nextToken'} : undef;
    }
    return $hReports;
}

#SPAPI - getReport -----------------------------------------------------------------
# see: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#get-reports2021-06-30reportsreportid
# resource path: GET /reports/2021-06-30/reports/{reportId}
sub getReport {
    my $self = shift;
    my ($ReportId) = @_;

    TestString( 'ReportId', $ReportId );

    my $URI = $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/reports/' . $ReportId;
    my $Response = $self->{PortalConfig}->doRequest('GET', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return from_json($Response->decoded_content);
}

#SPAPI - getReportDocument -----------------------------------------------------------------
# see: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#getreportdocument
# resource path: GET /reports/2021-06-30/documents/{reportDocumentId}
sub getReportDocument {
    my $self = shift;
    my ($ReportDocID) = @_;

    TestString( '$ReportDocID', $ReportDocID );

    my $Path = SP_API_REPORTS_RESSOURCE . '/documents/' . $ReportDocID;

    my $URI = $self->{SPAPI_EndPoint} . $Path;

    my $Response = $self->{PortalConfig}->doRequest('GET', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'RDTPath' => $Path, 'ThrottlingWaitTime' => 61});

    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    my $ResponseContent = decode_json($Response->decoded_content());
    my $DocumentDownloadUrl = $ResponseContent->{'url'};

    my $DocumentContent = GetHttpContent($DocumentDownloadUrl);

    return $DocumentContent;
}

sub _buildRequestURL{
    my ($APIUrl, $hQueryParams) = @_;

    my $ParamString = defined $hQueryParams ? BuildParameterString($hQueryParams) . '&' : '';
    $APIUrl .= "?$ParamString";

    return $APIUrl;
}

#SPAPI - GetMarketplaceParticipations -> getmarketplaceparticipations -----------------------------------------------------------------
# see: https://developer-docs.amazon.com/sp-api/docs/sellers-api-v1-reference#getmarketplaceparticipations
# resource path: GET /sellers/v1/marketplaceParticipations
# response:
# MarketPlacesIDs (List with all participating MarketPlaceIds)
sub getMarketPlaceParticipations {
    my $self = shift;

    my $URI = $self->{SPAPI_EndPoint} . SP_API_SELLERS_RESSOURCE . '/marketplaceParticipations';
    my $Response = $self->{PortalConfig}->doRequest('GET', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 64});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    my $DecodedResponse = decode_json($Response->decoded_content());
    my @MarketPlaceIDs = ();
    my $hMarketPlacesWhitelist = {
        'A2VIGQ35RCS4UG' => 1, # United Arab Emirates
        'A1PA6795UKMFR9' => 1, # Germany
        'ARBP9OOSHTCHU'  => 1, # Egypt
        'A1RKKUPIHCS9HS' => 1, # Spain
        'A13V1IB3VIYZZH' => 1, # France
        'AMEN7PMS3EDWL'  => 1, # Belgium
        'A1F83G8C2ARO7P' => 1, # UK
        'A21TJRUUN4KGV'  => 1, # India
        'APJ6JRA9NG5V4'  => 1, # Italy
        'A1805IZSGTT6HS' => 1, # Netherlands
        'A1C3SOZRARQ6R3' => 1, # Poland
        'A17E79C6D8DWNP' => 1, # Saudi Arabia
        'A2NODRKZP88ZB9' => 1, # Sweden
        'A33AVAJ2PDY3EV' => 1, # Turkey
    };

    if(defined $DecodedResponse->{'payload'}) {
        foreach my $MarketPlace (@{$DecodedResponse->{'payload'}}) {
            if (exists($MarketPlace->{'participation'}) && $MarketPlace->{'participation'}->{'isParticipating'}) {
                if ($hMarketPlacesWhitelist->{$MarketPlace->{'marketplace'}->{'id'}} == 1) {
                    push (@MarketPlaceIDs, $MarketPlace->{'marketplace'}->{'id'});
                }
            }
        }
    }

    return \@MarketPlaceIDs;
}

#SPAPI - ManageReportSchedule ->  createReportSchedule -----------------------------------------------------------------
# see: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#description-5
# resource path: POST /reports/2021-06-30/schedules

# required body: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#createreportschedulespecification
# reportType
# marketplaceIds []
# reportOptions (opt)
# period  (https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#period)   PT15M
# nextReportCreationTime (opt)
# response:
# reportScheduleId (The identifier for the report schedule. This identifier is unique only in combination with a seller ID.)
sub createReportSchedule {
    my $self = shift;
    my (%args) = @_;

    TestString( 'ReportType', $args{ReportType} );
    TestString( 'Schedule',   $args{Schedule} );

    my $aMarketPlaceIDs = $self->getMarketPlaceParticipations();

    my $hContent = {
        'marketplaceIds' => $aMarketPlaceIDs,
        'period'         => $args{Schedule},
        'reportType'     => $args{ReportType},
    };
    my $Response = $self->{PortalConfig}->doRequest('POST', $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/schedules', to_json($hContent), {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}

# required body: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#getReportSchedules
# reportType
# response:
# aReportScheduleIds (The identifier for the report schedule. This identifier is unique only in combination with a seller ID.)
sub getReportSchedulers {
    my $self = shift;
    my (%args) = @_;

    TestString( 'ReportType', $args{ReportType} );

    my $URI = $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/schedules?reportTypes=' . $args{ReportType};
    my $Response = $self->{PortalConfig}->doRequest('GET', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});

    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    my $DecodedResponse =  decode_json($Response->decoded_content());
    my @SchedulerIDs = ();
    foreach my $ReportSchedules (@ {$DecodedResponse->{'reportSchedules'}}) {
        push @SchedulerIDs, $ReportSchedules->{'reportScheduleId'};
    }

    return \@SchedulerIDs;
}

# required body: https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#deleteReportSchedulers
# $SchedulerID
sub deleteReportScheduler {
    my $self = shift;
    my ($SchedulerID) = @_;

    TestString( 'SchedulerID', $SchedulerID );

    my $URI = $self->{SPAPI_EndPoint} . SP_API_REPORTS_RESSOURCE . '/schedules/' . $SchedulerID;
    my $Response = $self->{PortalConfig}->doRequest('DELETE', $URI, '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});

    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    my $DecodedResponse =  decode_json($Response->decoded_content());
    my @SchedulerIDs = ();
    foreach my $ReportSchedules (@ {$DecodedResponse->{'reportSchedules'}}) {
        push @SchedulerIDs, $ReportSchedules->{'reportScheduleId'};
    }

    return \@SchedulerIDs;
}

# Creates a feed document for the feed type that you specify. This operation returns a presigned URL for uploading the feed document contents.
# It also returns a feedDocumentId value that you can pass in with a subsequent call to the createFeed operation.
# Attention this could be a bottleneck due to the Rate/Burst plan
# Standard Usage Plan: Rate (requests per second) 0.0083, Burst 15
sub createFeedDocument {
    my $self = shift;

    my $hContent = {
        'contentType' => 'text/xml; charset=UTF-8'
    };

    my $URI = $self->{SPAPI_EndPoint} . SP_API_FEEDS_RESSOURCE . '/documents';
    my $Response = $self->{PortalConfig}->doRequest('POST', $URI, to_json($hContent), {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 121});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}

# Upload the constructed feed data
# No documentation for the PUT call, we assume to have the standard throttling
sub uploadFeedData {
    my $self = shift;
    my ($Content, $FeedDocumentURL ) = @_;

    my $Response = $self->{PortalConfig}->doRequest('PUT', $FeedDocumentURL, $Content, {'Content-Type' => 'text/xml; charset=UTF-8'}, {'SignRequest'=>0, 'ThrottlingWaitTime' => 47});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return $Response->decoded_content();
}

# Attention this could be a bottleneck due to the Rate/Burst plan
# Standard Usage Plan: Rate (requests per second) 0.0083, Burst 15
sub createFeed {
    my $self = shift;
    my ($Content) = @_;

    my $URL = $self->{SPAPI_EndPoint} . SP_API_FEEDS_RESSOURCE . '/feeds';
    my $Response = $self->{PortalConfig}->doRequest('POST', $URL,  to_json($Content), {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 121});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}

# Returns feed details (including the resultDocumentId, if available) for the feed that you specify.
# Rate (requests per second) 2.0 Burst 15
sub getFeedProcessingReport {
    my $self = shift;
    my ($DocId) = @_;

    my $URL = $self->{SPAPI_EndPoint} . SP_API_FEEDS_RESSOURCE . '/feeds/'. $DocId;
    my $Response = $self->{PortalConfig}->doRequest('GET', $URL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 1});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}

# Returns the information required for retrieving a feed document's contents.
# Rate (requests per second) 0.0222 Burst 10
sub getFeedDocumentData {
    my $self = shift;
    my ($FeedDocumentID) = @_;

    my $URL = $self->{SPAPI_EndPoint} . SP_API_FEEDS_RESSOURCE . '/documents/'. $FeedDocumentID;
    my $Response = $self->{PortalConfig}->doRequest('GET', $URL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}


# Rate (requests per second) 0.0222 Burst 10
sub getFeedProcessingReportDocument {
    my $self = shift;
    my ($FeedDocumentURL) = @_;

    my $Response = $self->{PortalConfig}->doRequest('GET', $FeedDocumentURL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>0, 'ThrottlingWaitTime' => 47});
    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content }) unless $Response && $Response->is_success();

    return $Response->decoded_content();
}

# Step 1. Create a feed document
# Step 2. Construct a feed (XML feeds)
# Step 3. Upload the feed data
# Step 4. Create a feed
sub submitFeed {
    my $self = shift;
    my ($hInfo) = @_;

    TestString( 'FeedType', $hInfo->{FeedType} );
    Error( 'PARAMETER_MISSING', { 'Parameter' => 'Content' } ) unless defined $hInfo->{Content};

    # Creates a feed document for the feed type that you specify. This operation returns a presigned URL for uploading the feed document contents.
    # It also returns a feedDocumentId value that you can pass in with a subsequent call to the createFeed operation.
    my $FeedDocumentInfo = $self->createFeedDocument();
    my $FeedDocumentID = $FeedDocumentInfo->{'feedDocumentId'};
    my $FeedDocumentURL = $FeedDocumentInfo->{'url'};

    # Upload the constructed feed data
    $self->uploadFeedData($hInfo->{'Content'}, $FeedDocumentURL);

    my $hContent = { 'feedType' => $hInfo->{FeedType},
                     'marketplaceIds'        => [ $self->marketplaceID ],
                     'inputFeedDocumentId'   => $FeedDocumentID,
                   };

    my $hAmazonFeedId = $self->createFeed($hContent);

    return $hAmazonFeedId->{'feedId'};
}

# get Amazon Processing Status for the feed.
sub getFeedSubmissionList {
    my $self = shift;
    my ($FeedId) = @_;

    my $URL = $self->{SPAPI_EndPoint} . SP_API_FEEDS_RESSOURCE . '/feeds/'. $FeedId;
    my $Response = $self->{PortalConfig}->doRequest('GET', $URL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});
    my $CheckResponse = decode_json($Response->decoded_content());

    if($CheckResponse->{'errors'}[0]->{'code'} eq 'NotFound') {
        sleep(2);
        $Response = $self->{PortalConfig}->doRequest('GET', $URL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 47});
    }
    Error('API_CALL_ERROR', { 'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content, 'FeedID' => $FeedId}) unless $Response && $Response->is_success();

    return decode_json($Response->decoded_content());
}


sub getFeedSubmissionResult {
    my $self = shift;
    my (%args) = @_;


    TestString( 'FeedSubmissionId', $args{FeedSubmissionId} );

    my %request = (
        'Action'   => 'GetFeedSubmissionResult',
        'Merchant' => $self->merchantID,
        'MWSAuthToken' => $self->mwsAuthToken,
        %args,
    );
    return $self->query( \%request );
}

# Returns the lowest priced offers for a single item based on ASIN
# Rate (requests per second) 0.5 Burst 1
sub getLowestOfferList {
    my $self = shift;
    my (%args) = @_;
    my %hResponse = ();

    foreach my $Item (values %args){
        my $URL = $self->{SPAPI_EndPoint} .SP_API_PRODUCTS_PRICING_RESSOURCE . '/items/'. $Item->{'ASIN'} . '/offers'. '?MarketplaceId='. $Item->{'MarketplaceId'} . '&ItemCondition=' . $Item->{'ItemCondition'};
        my $Response = $self->{PortalConfig}->doRequest('GET', $URL,  '', {'Content-Type' => 'application/json'}, {'SignRequest'=>1, 'ThrottlingWaitTime' => 3});

        my $InvalidASIN;
        unless ($Response && $Response->is_success()) {
            if ($Response->code == 400) {
                my $ResponseErrorCode = from_json($Response->decoded_content)->{'errors'}[0]->{'code'};
                my $ResponseErrorMessage = from_json($Response->decoded_content)->{'errors'}[0]->{'message'};

                if ($ResponseErrorCode = 'InvalidInput' && $ResponseErrorMessage =~ /$Item->{'ASIN'} is an invalid ASIN for marketplace/) {
                    $InvalidASIN = 1;
                } else {
                    Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content });
                }
            } else {
                Error('API_CALL_ERROR', {'CallName' => (caller(0))[3], 'ResponseCode' => $Response->code, 'Content' => $Response->decoded_content });
            }
        }

        $hResponse{$Item->{'ASIN'}} =  { 'ASIN' => $Item->{'ASIN'},
            'LowestPrices'                      => from_json($Response->decoded_content)->{'payload'}->{'Summary'}->{'LowestPrices'},
            'InvalidASIN'                       => $InvalidASIN,
        };
    }



    return \%hResponse;
}

sub getOrderServiceStatus {
    my $self = shift;
    my (%args) = @_;

    my %request = (
        'Action'   => 'GetServiceStatus',
        'SellerId' => $self->merchantID,
        'MWSAuthToken' => $self->mwsAuthToken,
        'Version'  => '2013-09-01',
        %args,
    );

    my $Url = $self->awsUrl->clone;
    $Url->path('/Orders/2013-09-01');
    return $self->query( \%request, $Url );
}

sub marketplaceID {
    return shift->{'marketplaceID'};
}

sub merchantID {
    return shift->{'merchantID'};
}

sub mwsAuthToken {
    return shift->{'mwsAuthToken'};
}

sub secondaryMarketplaceIDs {
    return shift->{'secondaryMarketplaceIDs'};
}

sub setSecondaryMarketplaceIDs {
    my $self = shift;
    my ($SecondaryMarketplaceIDs) = @_;
    $self->{'secondaryMarketplaceIDs'} = $SecondaryMarketplaceIDs;
}

sub loggerKey {
    return 'RemoteMWS';
}

1;
