use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Klarna::API::KlarnaOrderStatus qw( CheckKlarnaOrderStatusViaScheduler );
use DE_EPAGES::Klarna::DAL::Klarna qw( DAL_GetPendingKlarnaOrdersForOpenShops DAL_GetPendingKlarnaOrdersByShopID );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadObject LoadObjectByPath );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use Getopt::Long;
use Time::HiRes qw ( time );

## no critic qw(ProhibitMagicNumbers)

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'         => \$Help,
        'nooutput'     => \$NoOutput,
        'passwd=s'     => \$Password,
        'storename=s'  => \$StoreName,
        'shop=s'       => \$ShopAlias,
    );

    usage() if $Help;
    usage() unless $StoreName;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {

            # make sure that the cartridge is installed
            unless ( ExistsCartridgeByPackage('DE_EPAGES::Klarna') ) {
                print 'Store: ' . $StoreName . ": Cartridge Klarna is not installed.\n" unless ($NoOutput);
                return;
            }
            _CheckKlarnaOrders({
                    'PrintOutput' => ($NoOutput) ? 0 : 1,
                    'StoreName'   => $StoreName,
                    'ShopAlias'   => $ShopAlias,
                }
            );
        },
    );
    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.2f seconds', $seconds );
    print $logmsg . "\n" unless ($NoOutput);

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

#========================================================================================
# function     Function
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CheckKlarnaOrder($Order, $NoOutput);
#----------------------------------------------------------------------------------------
# description  First, the function checks if the order is pending.
#               If yes, it checks if the order has a LineItemContainer. If not, an error message is printed out.
#               Next, the function checks if the LineItemContainer is a live LineItemPaymentKlarna object.
#               If it is, it stores the PaymentMethod and the Kr_InvoiceNo. If the Kr_InvoiceNo is not valid, an error message is printed out. 
#               If the Kr_InvoiceNo is valid, a message is printed out to update the Klarna invoice of the order.
#               The CheckKlarnaOrderStatusViaScheduler function is then called, with the Order object as a parameter.
#               If the result status code is not OK, an error message is printed out.
#               Finally, the function returns nothing.
#----------------------------------------------------------------------------------------
# input        $Order | order object | object
#========================================================================================
sub _CheckKlarnaOrder {
    my ($Order, $NoOutput) = @_;

    print 'Checking pending order ' . $Order->pathString . "...\n" unless($NoOutput);
    my $OrderPending = $Order->get('PendingOn');
    return unless($OrderPending);

    unless($Order->existsChild('LineItemContainer')) {
        print STDERR 'WARNING: Order ' . $Order->pathString . ' has no LineItemContainer!' . "\n" unless($NoOutput);
        return;
    }

    my $LineItemContainer = $Order->child('LineItemContainer');
    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless(defined($LineItemPayment) && $LineItemPayment->instanceOf('LineItemPaymentKlarna') && !$LineItemPayment->get('IsTestTransaction'));

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    return unless(defined($PaymentMethod));

    my $InvoiceNo = $LineItemPayment->get('Kr_InvoiceNo');
    unless(isValid($InvoiceNo)) {
        print STDERR 'WARNING: Order ' . $Order->pathString . ' is incomplete. Skipping...' . "\n" unless($NoOutput);
        return;
    }

    print("Updating Klarna invoice $InvoiceNo of order " . $Order->pathString . "...\n") unless($NoOutput);

    my $Result = CheckKlarnaOrderStatusViaScheduler($Order);
    if($Result->{'StatusCode'} ne 'OK') {
        print STDERR ("Error updating Klarna invoice $InvoiceNo of order " . $Order->pathString . ': (' .
            $Result->{'StatusCode'} . ') ' . $Result->{'Message'} . "\n") unless($NoOutput);
    }
    return;
}

#========================================================================================
# function     Function
# state        private
#----------------------------------------------------------------------------------------
# syntax       _CheckKlarnaOrders( $hInput );
#----------------------------------------------------------------------------------------
# description  This code is part of a larger subroutine that is used to check Klarna orders.
#               The code begins by taking in a hash of inputs, including a flag for whether to print output (NoOutput).
#               The code then checks if a ShopAlias is defined.
#               If it is, the code attempts to load the shop by its alias. If this fails, an error message is printed to STDERR.
#               Next, the code checks if a Shop is defined.
#               If it is, a list of pending Klarna orders is retrieved from the database using the shop's ID. If no Shop is defined, a list of pending Klarna orders is retrieved from the database for all open shops.
#               Finally, the code iterates through the list of orders, checking each one using the _CheckKlarnaOrder() subroutine.
#----------------------------------------------------------------------------------------
# input        $hInput | #TODO: Add input description for Function | refhash
#========================================================================================
sub _CheckKlarnaOrders {
    my ($hInput) = @_;
    my $NoOutput = $hInput->{'PrintOutput'} ? 0 : 1;

    my $Shop;
    if(defined($hInput->{'ShopAlias'} and $hInput->{'ShopAlias'} ne 'all')) {
        eval {$Shop = LoadObjectByPath('/Shops/'. $hInput->{'ShopAlias'}); };
        if ( ExistsError() ) {
            my $Error = GetError();
            printf STDERR ( "Invalid Shop: %s\n", substr( $Error->shortMessage, 0, 4096 ) )  unless($NoOutput);
            return;
        }
    }

    my $aOrderIDs;
    if (defined $Shop) {
        if ($Shop->get('IsInternalTestShop')) {
            return;
        }
        $aOrderIDs = DAL_GetPendingKlarnaOrdersByShopID($Shop->id);
    } else {
        $aOrderIDs = DAL_GetPendingKlarnaOrdersForOpenShops();
    }

    foreach my $OrderID (@$aOrderIDs) {
        next unless ExistsObject($OrderID);
        my $Order = LoadObject($OrderID);
        _CheckKlarnaOrder($Order, $NoOutput);
    }
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias             (optional)   (all)

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
