#========================================================================================
# package      DE_EPAGES::Ogone::API::Object::PaymentMethodOgone
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::Ogone::API::Object::PaymentMethodOgone;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Ogone::API::Constants qw ( @OGONE_CURRENCIES @ACCEPTANCE_CURRENCIES );
use DE_EPAGES::Ogone::API::Tools qw ( GetAddressEmail );

use strict;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns undef as no feature is connected to this PaymentMethod.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'Ogone'; }

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $hInfo = $self->get([qw( MerchantName MerchantPassword Passphrase MerchantNameTest MerchantPasswordTest PassphraseTest )]);
    if ( $self->get('IsInTestMode') ) {
        return  $hInfo->{'MerchantNameTest'} && $hInfo->{'MerchantPasswordTest'} && $hInfo->{'PassphraseTest'}
                && $self->SUPER::usableAtStorefront($hVars);
    } else {
        return  $hInfo->{'MerchantName'} && $hInfo->{'MerchantPassword'} && $hInfo->{'Passphrase'}
                && $self->SUPER::usableAtStorefront($hVars);
    }
}


#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP, CHF
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $PaymentType = $self->get('PaymentType');

    # check for supported currencies
    if ($PaymentType->alias() eq 'Acceptance') {
        return 0 unless ($hVars->{'CurrencyID'} ~~ [@ACCEPTANCE_CURRENCIES]);
    } else {
        return 0 unless ($hVars->{'CurrencyID'} ~~ [@OGONE_CURRENCIES]);
    }

    # check for payment method specifics
    # Postfinance-TWINT requires to send the EMail
    if ( $PaymentType->alias() eq 'Postfinance') {
        my $SplitGatewayMethod = $self->get('SplitGatewayMethod');
        if ($SplitGatewayMethod eq 'TWINT') {
            my $Basket = $hVars->{'LineItemContainer'}->parent();
            my $BillingAddress = $Basket->get('BillingAddress');
            if (isValid($BillingAddress)) { # this method is also called before adding an address to the basket
                my $Email = GetAddressEmail($BillingAddress);
                return 0 unless (isValid($Email));
            }
        }
    }

    return $self->SUPER::canAddToBasket($hVars);

}


#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Ogone::API::Object::LineItemPaymentOgone->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $self = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$self->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 1
    };
}

1;
