#========================================================================================
# package      DE_EPAGES::Ogone::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the Ogone global functionality.
#========================================================================================
package DE_EPAGES::Ogone::API::Tools;
use base qw ( Exporter );

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use Digest::SHA1  qw (sha1_hex);
use Encode qw ( encode_utf8 );

our @EXPORT_OK = qw ( Ogone_SHA1 GetAddressEmail );

#========================================================================================
# function    Ogone_SHA1
# state       public
#----------------------------------------------------------------------------------------
# syntax      $key = Ogone_SHA1($AMOUNT, $BRAND, $CURRENCY, $LANGUAGE, $ORDERID,
#                                   $PM, $PSPID);
#----------------------------------------------------------------------------------------
# description calculate Ogone SHA1 hash for request
#----------------------------------------------------------------------------------------
# input       $passphrase | Passphrase | string
# input       $hFormValues | hash of input string values | ref.hash
# input       $aParameters | array of valid form parameters | ref.array
# return      SHA1 checksum in upper case
#========================================================================================
sub Ogone_SHA1 {
    my ($passphrase, $hFormValues, $aParameters) = @_;

    # hack: we may get 'Currency' and 'currency' paramenters that may differ. The one that applies to the ShaSign
    # calculation is the 'currency'
    my @formKeys = keys(%$hFormValues);
    my $index = 0;
    foreach my $key (@formKeys) {
        last if ($key eq 'Currency');
        $index++
    };
    splice(@formKeys, $index, 1) if ($index <= $#formKeys);

    my %normalizedFormValues = map { uc($_) => $hFormValues->{$_} } @formKeys;

    # if there's no parameters array, form hash parameters should fit with SHA signature parameters

    $aParameters = [sort(@formKeys)] unless (defined $aParameters);

    my @aData = map { $_ . '=' . ($normalizedFormValues{$_} // '') } @$aParameters;
    # string to calculate the hash from
    my @aDataValid = ();
    foreach my $data (@aData) {
        push(@aDataValid, $data) if ($data !~ /[A-Z_]+=$/);
    }
    push(@aDataValid, '');

    my $stringToDigest = join($passphrase, map {$_ // ''} @aDataValid); # param=value

    my $digest = uc(sha1_hex(encode_utf8($stringToDigest)));

    # we replace the Passphrase from the log line !
    my $qPassphrase = quotemeta($passphrase);

    my $digestToLog = $stringToDigest;
    $digestToLog =~ s/$qPassphrase/{PASSPHRASE}/g;

    LogPayment(
        'Ogone',
        "digest for string\n$digestToLog\n",
        $digest
    ) ;

    return $digest;
}

#========================================================================================
# function    GetAddressEmail
# state       public
#----------------------------------------------------------------------------------------
# syntax      $Email = GetAddressEmail($Address);
#----------------------------------------------------------------------------------------
# description get one of the emails from address, ordered by EMail, EMailBusiness or EMailPrivate
#----------------------------------------------------------------------------------------
# input       $Address | address object | object
# return      $Email | email string | string
#========================================================================================
sub GetAddressEmail {
    my ($Address) = @_;

    my $hAddressEmails = $Address->get([qw(EMail EMailBusiness EMailPrivate)]);

    if (isValid($hAddressEmails->{'EMail'})) {
        return $hAddressEmails->{'EMail'}
    }
    if (isValid($hAddressEmails->{'EMailBusiness'})) {
        return $hAddressEmails->{'EMailBusiness'}
    }
    if (isValid($hAddressEmails->{'EMailPrivate'})) {
        return $hAddressEmails->{'EMailPrivate'}
    }
    return;
}

1;
