#========================================================================================
# package      DE_EPAGES::Order::API::XML::CustomerOrderExport
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides functions to export orders to a xml file.
#========================================================================================
package  DE_EPAGES::Order::API::XML::CustomerOrderExport;
use base DE_EPAGES::Presentation::API::XML::MonitoredExportDriver;

use strict;

use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Currency qw (GetInfoCurrency);

#========================================================================================
# function     drive
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  Starts the monitor and calls addElement of each object
#========================================================================================
sub drive {
    my $self = shift;

    $self->startMonitor('Monitor') if exists $self->{'Monitor'};
    $self->_addHooks();
    $self->addElement('Orders', {});
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aOrders = $self->items;
#----------------------------------------------------------------------------------------
# description  Returns all orders for the export, initializes monitor.
#----------------------------------------------------------------------------------------
# return       $aOrders | list of order objects | ref.array.object
#========================================================================================
sub items {
    my $self = shift;

    my $aItems = $self->SUPER::items();
    if (exists $self->{'Monitor'}) {
        my $Monitor = $self->monitor;
        $Monitor->set({'Total'=> scalar @$aItems}) if defined $Monitor;
    }
    return $aItems;
}

sub _addHooks {
    my $self = shift;

    $self->addHook('Orders' => 'GetShop');
    $self->addHook('Shop'   => 'GetShopGUID');
    $self->addHook('Orders' => 'GetOrders');
    $self->addHook('Order' => 'GetOrderNumber');
    $self->addHook('Order' => 'GetCustomerNumber');
    $self->addHook('Order' => 'GetInternalComment');
    $self->addHook('Order' => 'GetCurrency');
    $self->addHook('Order' => 'GetLanguage');
    $self->addHook('Order' => 'GetLocale');
    $self->addHook('Order' => 'GetTaxArea');
    $self->addHook('Order' => 'GetTaxModel');
    $self->addHook('Order' => 'GetGrandTotal');
    $self->addHook('Order' => 'GetTotalBeforeTax');
    $self->addHook('Order' => 'GetTotalTax');
    $self->addHook('Order' => 'GetAddresses');
    $self->addHook('Order' => 'GetLineItems');
    $self->addHook('Order' => 'GetEcoParticipationCategories' );
    $self->addHook('Order' => 'GetAttributes' );
    $self->addHook('Order' => 'GetOrderDocuments');
    $self->addHook('OrderDocuments' => 'GetInvoices' );
    $self->addHook('OrderDocuments' => 'GetCreditNotes' );
    $self->addHook('OrderDocuments' => 'GetPackingSlips' );
    $self->addHook('Invoices'       => 'GetInvoice');
    $self->addHook('PackingSlips'   => 'GetPackingSlip');
    $self->addHook('CreditNotes'    => 'GetCreditNote');
    $self->addHook('Invoice'        => 'GetInvoiceDetails');
    $self->addHook('PackingSlip'    => 'GetPackingSlipDetails');
    $self->addHook('CreditNote'     => 'GetCreditNoteDetails');
    $self->addHook('Addresses' => 'GetBillingAddress');
    $self->addHook('Addresses' => 'GetShippingAddress');
    $self->addHook('BillingAddress' => 'GetAddressAttributes' );
    $self->addHook('BillingAddress' => 'GetCountry');
    $self->addHook('ShippingAddress' => 'GetAddressAttributes' );
    $self->addHook('ShippingAddress' => 'GetCountry');
    $self->addHook('LineItems' => 'GetLineItem');
    $self->addHook('LineItem' => 'GetIdentifier');
    $self->addHook('LineItem' => 'GetName');
    $self->addHook('LineItem' => 'GetQuantity');
    $self->addHook('LineItem' => 'GetOrderUnit');
    $self->addHook('LineItem' => 'GetUnitPrice');
    $self->addHook('LineItem' => 'GetDiscount');
    $self->addHook('LineItem' => 'GetTotalPrice');
    $self->addHook('LineItem' => 'GetTaxRate');
    $self->addHook('LineItem' => 'GetAttributes' );
    $self->addHook('LineItem' => 'GetComment' );
    $self->addHook('LineItem' => 'GetCustomizedText' );
    $self->addHook('LineItemSalesDiscount' => 'GetIdentifier');
    $self->addHook('LineItemSalesDiscount' => 'GetTotalPrice');
    $self->addHook('LineItemSalesDiscount' => 'GetIsShippingIncluded');
    $self->addHook('LineItemSalesDiscount' => 'GetAttributes' );
    $self->addHook('LineItemPrePayment' => 'GetIdentifier');
    $self->addHook('LineItemPrePayment' => 'GetTotalPrice');
    $self->addHook('LineItemPrePayment' => 'GetAttributes' );
    $self->addHook('LineItemDiscount' => 'GetTotalPrice');
    $self->addHook('LineItemDiscount' => 'GetAttributes' );
    $self->addHook('LineItemShipping' => 'GetShippingIdentifier');
    $self->addHook('LineItemShipping' => 'GetDiscount');
    $self->addHook('LineItemShipping' => 'GetTotalPrice');
    $self->addHook('LineItemShipping' => 'GetAttributes' );
    $self->addHook('LineItemShipping' => 'GetTaxComponents' );
    $self->addHook('LineItemPayment' => 'GetPaymentIdentifier');
    $self->addHook('LineItemPayment' => 'GetDiscount');
    $self->addHook('LineItemPayment' => 'GetTotalPrice');
    $self->addHook('LineItemPayment' => 'GetAttributes' );
    $self->addHook('LineItemPayment' => 'GetTaxComponents' );
    $self->addHook('LineItemPaymentDiscount' => 'GetTotalPrice');
    $self->addHook('LineItemPaymentDiscount' => 'GetAttributes' );
    $self->addHook('LineItemEcoParticipation' => 'GetTotalPrice');
    $self->addHook('LineItemEcoParticipation' => 'GetAttributes' );
    $self->addHook('TaxComponent' => 'GetTaxComponent' );
}

sub formatPrice {
    my $self = shift;
    my ($Container, $Price) = @_;
    return unless defined $Price;

    my $CurrencyID = $Container->get('CurrencyID');
    my $Digits = GetInfoCurrency($CurrencyID)->{'MinorUnit'};
    return $Digits ? sprintf("%0.${Digits}f", $Price) : $Price;
}

sub getPrice {
    my $self = shift;
    my ($Container, $Object, $Attribute, $Name) = @_;
    my $Price = $Object->get($Attribute);
    $self->addElement($Name, {}, $self->formatPrice($Container, $Price)) if defined $Price;
}

sub languageID {
    my $self = shift;
    my ($Container) = @_;
    $self->{'LanguageID'} = defined $Container ? $Container->get('LanguageID') : undef unless defined $self->{'LanguageID'};
    return $self->{'LanguageID'};
}

sub Orders { return {}; }

sub GetOrders {
    my $self = shift;

    $self->addElement( 'Order', {
        'Object' => $_,
    }) foreach @{$self->items};
}

sub GetShop {
    my $self = shift;
    $self->addElement( 'Shop', {'Object' => $self->object});
}

sub GetShopGUID {
    my $self = shift;
    my $Shop = $self->topID('Object');
    $self->addElement('GUID', {}, $Shop->get('GUID'));
}

sub Order {
    my $self = shift;
    $self->monitorIncrement;
    $self->monitorContent;
    return {};
}

sub GetOrderNumber {
    my $self = shift;
    my $Order = $self->topID('Object');
    $self->addElement('OrderNumber', {}, $Order->alias);
}

sub GetCustomerNumber {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Customer = $Order->get('Customer');
    $self->addElement('CustomerNumber', {}, $Customer->alias) if defined $Customer;
}

sub GetInternalComment {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Comment = $Order->get('Comment');
    $self->addElement('InternalComment', {}, $Comment) if $Comment;
}

sub GetCurrency {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->addElement('Currency', {}, $Container->get('CurrencyID'));
}

sub GetLanguage {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->addElement('Language', {}, GetCodeByLanguageID($self->languageID($Container)));
}

sub GetLocale {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->addElement('Locale', {}, $Container->get('LocaleID'));
}

sub GetTaxArea {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    my $TaxArea = $Container->get('TaxArea');
    $self->addElement('TaxArea', {}, $TaxArea->alias) if defined $TaxArea;
}

sub GetTaxModel {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->addElement('TaxModel', {}, $Container->get('TaxModel') ? 'gross' : 'net');
}

sub GetGrandTotal {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->getPrice($Container, $Container, 'GrandTotal', 'GrandTotal');
}

sub GetTotalBeforeTax {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->getPrice($Container, $Container, 'TotalBeforeTax', 'TotalBeforeTax');
}

sub GetTotalTax {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->getPrice($Container, $Container, 'TotalTax', 'TotalTax');
}

sub GetAddresses {
    my $self = shift;
    $self->addElement('Addresses', {'Object' => $self->topID('Object')});
}

sub Addresses { return {}; }

sub GetBillingAddress {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $BillingAddress = $Order->get('BillingAddress');
    $self->addElement('BillingAddress', {'Object' => $BillingAddress});
}

sub BillingAddress { return {}; }

sub GetShippingAddress {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $ShippingAddress = $Order->get('ShippingAddress');
    $self->addElement('ShippingAddress', {'Object' => $ShippingAddress}) if defined $ShippingAddress;
}

sub ShippingAddress { return {}; }

sub GetAddressAttributes {
    my $self = shift;
    my $Address = $self->topID('Object');

    my @AddressAttributes = qw( Company Department JobTitle
                                FirstName MiddleName LastName Salutation Title Gender Birthday
                                AddressExtension Street Street2 CodePorte Zipcode State City FiscalCode
                                EMail EMailPrivate EMailBusiness
                                Phone PhonePrivate PhoneBusiness PhoneCell Fax
                                VATID URL
                              );
    foreach my $Attribute (@AddressAttributes) {
        my $Value = $Address->get($Attribute);
        $self->addElement($Attribute, {}, $Value) if $Value;
    }
}

sub GetCountry {
    my $self = shift;
    my $Address = $self->topID('Object');
    my $hCountry = $Address->get('Country');
    $self->addElement('Country', {}, $hCountry->{Code2}) if defined $hCountry;
}

sub GetLineItems {
    my $self = shift;
    $self->addElement('LineItems', {'Object' => $self->topID('Object')});
}

sub LineItems { return {}; }

sub GetLineItem {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;
    $self->addElement('LineItem', { 'Object' => $_, 'Container' => $Container }) foreach @{$Container->get('LineItems')};
    $self->addElement('LineItemSalesDiscount', { 'Object' => $_, 'Container' => $Container }) foreach @{$Container->get('SalesDiscounts')};
    $self->addElement('LineItemPrePayment', { 'Object' => $_, 'Container' => $Container }) foreach @{$Container->get('PrePayments')};
    $self->addElement('LineItemDiscount', { 'Object' => $_, 'Container' => $Container }) foreach @{$Container->get('Discounts')};
    my $Shipping = $Container->get('Shipping');
    $self->addElement('LineItemShipping', { 'Object' => $Shipping, 'Container' => $Container }) if defined $Shipping;
    my $Payment = $Container->get('Payment');
    $self->addElement('LineItemPayment', { 'Object' => $Payment, 'Container' => $Container }) if defined $Payment;
    my $PaymentDiscount = $Container->get('PaymentDiscount');
    $self->addElement('LineItemPaymentDiscount', { 'Object' => $PaymentDiscount, 'Container' => $Container }) if defined $PaymentDiscount;
    $self->addElement('LineItemEcoParticipation', { 'Object' => $_, 'Container' => $Container })
        foreach grep { $_->instanceOf('LineItemEcoParticipation') } @{$Container->children};
}

sub GetIdentifier {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $ID = $LineItem->get('SKU');
    $self->addElement('Id', {}, $ID) if defined $ID;
}

sub GetIsShippingIncluded {
    my $self = shift;
    my $LineItem        = $self->topID('Object');
    my $LineItemClass   = $LineItem->get('Class');
    $self->addElement('IsShippingIncluded', {}, $LineItem->get('InclShippingCosts')) if  $LineItemClass->existsAttribute('InclShippingCosts');
}

sub GetPaymentIdentifier {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $PaymentMethod = $LineItem->get('PaymentMethod');
    $self->addElement('Id', {}, $PaymentMethod->alias) if defined $PaymentMethod;
}

sub GetShippingIdentifier {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $ShippingMethod = $LineItem->get('ShippingMethod');
    $self->addElement('Id', {}, $ShippingMethod->alias) if defined $ShippingMethod;
}

sub GetTaxComponents {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $aTaxes = $LineItem->get('Taxes');

    foreach my $TaxComponent (@$aTaxes) {
        my $Postfix = defined $TaxComponent->{'TaxRateDigital'} ? 'Digital' : '';
        if ($TaxComponent->{'Value'} != 0) {
            $self->addElement('TaxComponent', {
                'TaxRate'. $Postfix => $TaxComponent->{'TaxRate' . $Postfix},
                'Value' => $TaxComponent->{'Value'}
            })
        };
    }
}

sub GetTaxComponent {
    my $self = shift;
    my $TaxRate = $self->topID('TaxRate');
    my $TaxRateDigital = $self->topID('TaxRateDigital');
    my $Value = $self->topID('Value');
    my $Postfix = defined $TaxRateDigital ? 'Digital' : '';

    $self->addElement('TaxRate' . $Postfix, {}, $TaxRateDigital // $TaxRate);
    $self->addElement('Value', {}, $Value);

}


sub GetName {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $Container = $self->topID('Container');
    my $Name = $LineItem->get('Name', $self->languageID($Container));
    $self->addElement('Name', {}, $Name) if defined $Name;
}

sub GetQuantity {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    $self->addElement('Quantity', {}, $LineItem->get('Quantity'));
}

sub GetOrderUnit {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $OrderUnit = $LineItem->class->existsAttribute('OrderUnit') ? $LineItem->get('OrderUnit') : undef;
    $self->addElement('OrderUnit', {}, $OrderUnit->alias) if defined $OrderUnit;
}

sub GetUnitPrice {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $Container = $self->topID('Container');
    $self->getPrice($Container, $LineItem, 'BasePrice', 'UnitPrice');
}

sub GetDiscount {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $Container = $self->topID('Container');
    $self->getPrice($Container, $LineItem, 'Discount', 'Discount');
}

sub GetTotalPrice {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    my $Container = $self->topID('Container');
    $self->getPrice($Container, $LineItem, 'LineItemPrice', 'TotalPrice');
}

sub GetTaxRate {
    my $self = shift;
    my $LineItem = $self->topID('Object');
    $self->addElement('TaxRate', {}, $LineItem->get('TaxRate'));
}

sub GetAttributes {
    my $self = shift;
    my $Object = $self->topID('Object');
    my $Container = $self->topID('Container');

    my $aAttributes = $Object->can('getExtraExportableAttributes') ? $Object->getExtraExportableAttributes : [];
    foreach my $Attribute (@$aAttributes) {
        my $Value = $Object->get($Attribute, $self->languageID($Container));
        $self->addElement($Attribute, {}, $Value) if $Value;
    }

    my $aSubContainers = $Object->can('getExtraContainerAttributes') ? $Object->getExtraContainerAttributes : [];
    foreach my $Attribute (@$aSubContainers) {
        my $SubContainer = $Object->get($Attribute);
        $self->addElement('LineItem', { 'Object' => $_, 'Container' => $SubContainer }) foreach @{$SubContainer->get('LineItems')};
    }
}

sub GetComment {
    my $self = shift;
    my $LineItem = $self->topID('Object');

    my $Comment = $LineItem->class->existsAttribute('Comment') ? $LineItem->get('Comment') : undef;
    $self->addElement('Comment', {}, $Comment) if defined $Comment;
}

sub GetCustomizedText {
    my $self = shift;
    my $LineItem = $self->topID('Object');

    my $CustomizedTextUnity = $LineItem->class->existsAttribute('CustomizedTextUnity') ? $LineItem->get('CustomizedTextUnity') : undef;
    $self->addElement('CustomizedText', {}, $CustomizedTextUnity) if defined $CustomizedTextUnity;
}

sub GetEcoParticipationCategories {
    my $self = shift;
    my $Order = $self->topID('Object');
    my $Container = $Order->container;

    return {} unless $Container->class->existsAttribute('EcoParticipationCategories');

    my $aCategories = $Container->get('EcoParticipationCategories');
    foreach my $hCategory (@$aCategories) {
        $self->addElement('EcoParticipationCategory', {
            'Category' => $hCategory->{'Category'}->get('NameOrAlias', $self->languageID($Container)),
            'Price' => $hCategory->{'Price'}
        });
    }
}

sub EcoParticipationCategory {
    my $self = shift;
    return {'Category' => $self->topID('Category'), 'Price' => $self->topID('Price')};
}


sub GetOrderDocuments {
    my $self = shift;
    my $OrderDocuments = $self->topID('Object')->get('Documents');
    $self->addElement('OrderDocuments', {'OrderDocuments' => $OrderDocuments}) if scalar @$OrderDocuments;
}

sub OrderDocuments { return {}; }

sub GetInvoices {
    my $self = shift;
    my $Invoices = [ grep { $_->instanceOf('Invoice') } @{$self->topID('OrderDocuments')} ] ;
    $self->addElement('Invoices', {'Invoices' => $Invoices}) if scalar @$Invoices;
}

sub GetPackingSlips {
    my $self = shift;
    my $PackingSlips = [ grep { $_->instanceOf('PackingSlip') } @{$self->topID('OrderDocuments')} ] ;
    $self->addElement('PackingSlips', {'PackingSlips' => $PackingSlips}) if scalar @$PackingSlips;

}

sub GetCreditNotes {
    my $self = shift;
    my $CreditNotes = [ grep { $_->instanceOf('CreditNote') } @{$self->topID('OrderDocuments')} ] ;
    $self->addElement('CreditNotes', {'CreditNotes' => $CreditNotes}) if scalar @$CreditNotes;

}

sub Invoices { return {}; }
sub PackingSlips { return {}; }
sub CreditNotes { return {}; }

sub GetInvoice {
    my $self = shift;
    my $aInvoices = $self->topID('Invoices');
    $self->addElement('Invoice', { 'Object' => $_ }) foreach @{$aInvoices};
}

sub GetInvoiceDetails {
    my $self = shift;
    my $Invoice = $self->topID('Object');
    $self->addElement('InvoiceNumber', {}, $Invoice->get('Alias'));
}

sub GetPackingSlip {
    my $self = shift;
    $self->addElement('PackingSlip', {'Object' => $self->topID('Object')});
}

sub GetPackingSlipDetails {
    my $self = shift;
    my $PackingSlip = $self->topID('Object');
    $self->addElement('PackingSlipNumber', {}, $PackingSlip->get('Alias'));
}

sub GetCreditNote {
    my $self = shift;
    $self->addElement('CreditNote', {'Object' => $self->topID('Object')});
}

sub GetCreditNoteDetails {
    my $self = shift;
    my $CreditNote = $self->topID('Object');
    $self->addElement('CreditNoteNumber', {}, $CreditNote->get('Alias'));
}


1;
