#========================================================================================
# package      DE_EPAGES::Product::API::CSV::ProductExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to export , import products
#========================================================================================
package DE_EPAGES::Product::API::CSV::ProductExportImport;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObject);
use DE_EPAGES::Object::API::ObjectArray qw (ObjectIterator);
use DE_EPAGES::Object::API::Table::Locale qw(GetInfoLocale);
use DE_EPAGES::Product::API::CSV::ProductExportDriver;
use DE_EPAGES::Product::API::CSV::ProductImportHandler;
use DE_EPAGES::Product::API::Constants qw( CSV_SPECIAL_PRODUCTITEMS CSV_SPECIAL_PRODUCTITEMS_UNITY_EXP CSV_SPECIAL_PRODUCTITEMS_UNITY_IMP CSV_WARNING_CODES_PRODUCT );
use DE_EPAGES::Product::DAL::Product qw( DAL_GetAllProductIDsAliasOrderedByShopID );
use DE_EPAGES::Shop::API::Constants qw( TaxModel2Text ) ;

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    my $self = shift;
    return [ @{$self->SUPER::warningCodes}, CSV_WARNING_CODES_PRODUCT ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::Product::API::CSV::ProductExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::Product::API::CSV::ProductImportHandler';
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ExportProducts = $Driver->getExportObjects($Shop, $aObjectIDs);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjectIDs     | list of exportable objects | ref.array.integer
# return       $ExportProducts | iterator of products to be exported | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjectIDs) = @_;

    if(not defined $aObjectIDs){ #get all products

        #sort by alias but superproduct should be at first position
        my $Iterator = DE_EPAGES::Core::API::Iterator->wrap(
            Source => DAL_GetAllProductIDsAliasOrderedByShopID($Shop->id),
            Handle => sub {
                LoadObject( $_[1]->[0] ); # my $self = shift; my ($aRow) = @_; my $ObjectID = $aRow->[0]; LoadObject($ObjectID);
            }
        );
        Warning('NO_PRODUCTS') unless $Iterator->count;
        return $Iterator;
    }
    else {

        my @ProductIDs; #export superproduct at first position
        my %IncludedProductIDs; # has version of @ProductIDs for exists() check
        foreach my $ProductID (@$aObjectIDs){

            # test if we have a ref otherwise load the object
            my $Product = ref $ProductID ? $ProductID : LoadObject($ProductID);
            push @ProductIDs, $Product->id;
            $IncludedProductIDs{$Product->id} = undef;
            foreach my $SubProductID (@{ $Product->findSubProducts }){
                #register each subproduct in $aObjectIDs for export
                unless ( exists $IncludedProductIDs{$SubProductID} ) {
                    push @ProductIDs, $SubProductID;
                    $IncludedProductIDs{$SubProductID} = undef;
                }
            }
        }
        Warning('NO_PRODUCTS') if 0 == scalar @ProductIDs;
        return ObjectIterator( \@ProductIDs );
    }
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s,$IsImport);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# input        $IsImport | true if this is an import ? | boolean
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s, $IsImport) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);
    my @Currencies = map {$_->{'CurrencyID'}} @{$Shop->get('Currencies')}; # EUR,USD

    my $TaxModel = $Shop->get('InputTaxModel') ;
    my $TaxModelText = TaxModel2Text($TaxModel);

    my @Attributes = sort { $a->get('Position') <=> $b->get('Position') }
        @{LoadClassByAlias('Product')->attributes};

    my $aProductTypes = $Shop->get('ProductTypes')->children;
    my %ProductTypes = map { $_->id => $_ } @$aProductTypes;
    push @Attributes, grep { $_->get('IsUserDefined') }
        map { sort { $a->get('Position') <=> $b->get('Position') } @{$_->directAttributes} }
        ($Shop->get('ProductClass'), @$aProductTypes);

    my @CSVAttributes;
    foreach my $Attribute ( @Attributes ) {
        next unless IsAttributeForCSV($Attribute, $Shop, $IsImport);
        my %Info;
        my $Alias = $Attribute->alias;
        if ( $Alias eq 'ListPrices' or $Alias eq 'ManufacturerPrices' or $Alias eq 'EcoParticipations' or $Alias eq 'Deposits') {
            $Info{'Type'} = 'Price';
            $Info{'IsObject'} = 0;
        } elsif ( $Alias eq 'SelectedVariations') {
            $Info{'Type'} = 'VariationAttributes';
            $Info{'IsObject'} = 1;
        } elsif ( $Alias eq 'ShippingMethods') {
            $Info{'Type'} = 'ShippingMethod';
            $Info{'IsObject'} = 1;
        } else {
            $Info{$_} = $Attribute->get($_) foreach qw (Type IsObject);
        }

        my $AttributeClass =  $Attribute->get('AttributeClass');
        if ( exists $ProductTypes{$AttributeClass->id} ) {
            $Info{'IsBaseClassAttribute'} = 0;
            $Info{'ClassAlias'} = $AttributeClass->alias;
        } else {
            $Info{'IsBaseClassAttribute'} = 1;
            $Info{'ClassAlias'} = 'ProductClass';
        }

        if ($Info{'Type'} eq 'Price') {
            my $hInfo = $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages, \%Info)->[0];
            foreach my $CurrencyID ( @Currencies) {
                my %AttributeInfo = %$hInfo;
                $AttributeInfo{'CurrencyID'} = $CurrencyID;
                my $hHeaderStrings = $self->getHeaderStrings( \%AttributeInfo, $hLocale->{'LanguageID'}, {
                    'CurrencyID'  => $CurrencyID,
                    'TaxModelText'=> $TaxModelText,
                } );
                $AttributeInfo{'HeaderString'} = $hHeaderStrings->{'HeaderString'};
                $AttributeInfo{'AliasString'}  = $hHeaderStrings->{'AliasString'};
                $AttributeInfo{'TaxModel'}     = $TaxModel;
                push @CSVAttributes, \%AttributeInfo;
            }

        # convert HeaderString to 'ID [UnityAlias]' for signing a Unity export file
        # with this information we can handle import files for Base and Now in a different way
        } elsif(($Alias eq 'Alias') and $Shop->get('IsUnityRegistered') ){ 
            my $hAttributeInfo = $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages, \%Info)->[0];
            $hAttributeInfo->{'HeaderString'} =~ s/\[Alias\]/\[UnityAlias\]/g;
            push @CSVAttributes, $hAttributeInfo;
        } else {
            push @CSVAttributes, @{ $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages, \%Info)};

        }
    }

    return \@CSVAttributes;
}

sub IsAttributeForCSV {

    my ($Attribute, $Shop, $IsImport) = @_;
    my $Alias = $Attribute->alias;
    # decide which attribute are importable and exportable for an unity or normal epages6 shop
    # if we have an import we are using the unity list for getting importable attributes. these attributes are set to default values later

    #Unity: Import and export
    if($Shop->get('IsUnityRegistered')) {
        #export
        if(not $IsImport){
            return 0 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS_UNITY_EXP->{0}});
            return 1 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS_UNITY_EXP->{1}});
        } else{
        #import
            return 0 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS_UNITY_IMP->{0}});
            return 1 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS_UNITY_IMP->{1}});

        }
    # base import / export
    } else {
        return 0 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS->{0}});
        return 1 if ElementOf($Alias, @{CSV_SPECIAL_PRODUCTITEMS->{1}}) ;
    }

    return 0 unless $Attribute->get('IsExportable');
    return 0 if $Attribute->get('Type') eq 'PreDefCustomerString';
    return 1 if $Attribute->get('Type') =~ /^PreDefMulti(Localized)?String$/;
    return 0 if ($Alias eq 'FilePathPart' and not $Shop->get('IsFilePathPartUsed'));
    return 0 if $Attribute->get('IsArray');
    return 1;
}

1;
