#========================================================================================
# package      DE_EPAGES::ServiRed::API::Object::LineItemPaymentBizum
# state        public
# base         DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
# description  object interface for LineItemPaymentBizum.
#========================================================================================
package DE_EPAGES::ServiRed::API::Object::LineItemPaymentBizum;
use base DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::ServiRed::API::Constants qw (STATUS_NEW);
use DE_EPAGES::Core::API::Error qw (Error);

use constant Bizum_MAX_ORDER_LENGTH => 12;

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemPaymentBizum->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className { 'LineItemPaymentBizum' }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentBizum = DE_EPAGES::ServiRed::API::Object::LineItemPaymentBizum->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentBizum');
#               my $LineItemPaymentBizum = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentBizum' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentBizum | new object | object
# hook         OBJ_InsertLineItemPaymentBizum | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentBizumID - line item payment id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemPaymentBizum', {'Object' => $self, 'LineItemPaymentBizumID' => $self->id });
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentBizum->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentBizum' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentBizum | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentBizumID - line item payment id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentBizum', {'Object' => $self, 'LineItemPaymentBizumID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentBizum->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentBizum' and 'OBJ_AfterUpdateLineItemPaymentBizum'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentBizum | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentBizumID - line item payment id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentBizum | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentBizumID - line item payment id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#----------------------------------------------------------------------------------------
# input        $hValues | hash of values to be set | ref.hash
# input        $LanguageID | language ID number | integer
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentBizum', {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentBizumID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemPaymentBizum',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentBizumID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ServiRed::API::Object::LineItemPaymentBizum->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $self = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$self->SUPER::default($Parent, $hInfo)},
        'TransStatus' => STATUS_NEW
    };
}

#========================================================================================
# function     updateOnExecute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentLineItem->updateOnExecute;
#----------------------------------------------------------------------------------------
# description  overwrite the function from the super class
#----------------------------------------------------------------------------------------
# input        $hParams | hash of parameters | ref.hash
#========================================================================================
sub updateOnExecute {
    my $self = shift;
    my ($hParams)= @_;

    $self->SUPER::updateOnExecute($hParams);

    my $TransIdPrefix = defined $hParams->{'Order'} ? $hParams->{'Order'}->alias() : $self->container()->id();
    $self->updateTransIdFromPrefix($TransIdPrefix);
}

#========================================================================================
# function     updateTransIdFromPrefix
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentLineItem->updateTransIdFromPrefix($TransIdPrefix);
#----------------------------------------------------------------------------------------
# description  takes a prefix as an argument and sets the TransactionType and TransID 
#               using a modified version of the prefix and the current time.
#----------------------------------------------------------------------------------------
# input        $TransID | order alias or container id | integer
#========================================================================================
sub updateTransIdFromPrefix {
    my $self = shift;
    my ($TransID)= @_;

    $TransID =~ s/[\D]//g;
    $TransID .= join('', reverse(split(//, time)));

    if (length($TransID) > Bizum_MAX_ORDER_LENGTH) {
        $TransID = substr($TransID, 0, Bizum_MAX_ORDER_LENGTH);
    }

    $self->set({
        'TransactionType' => $self->get('PaymentMethod')->get('TransactionType'),
        'TransID'         => $TransID
    });
}

#========================================================================================
# function     executePayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentBizum->executePayment($hParams);
#----------------------------------------------------------------------------------------
# description  This function directs the payment process.
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object (optional)</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub executePayment {
    my $self        = shift;
    my ($hParams)   = @_;

    $self->updateOnExecute($hParams);

    my $ViewAction = $hParams->{'Servlet'}->vars('IsMobile') ? 'MobileSF-ViewPaymentServiRed' : 'ViewPaymentServiRed';
    # show payment template
    my $Object = $hParams->{'Order'} // $hParams->{'Basket'};
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $Object->id,
        },
        'ViewAction' => $ViewAction
    });

    return;
}

1;

