#========================================================================================
# package      DE_EPAGES::ShoppingList::API::Object::ShoppingList
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ShoppingList.
#========================================================================================
package DE_EPAGES::ShoppingList::API::Object::ShoppingList;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DateTime;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Shop::API::Constants qw ( BASKETACTION_ADDQUANTITY BASKETACTION_REPLACEQUANTITY
                                        BASKETACTION_DONTCHANGEQUANTITY BASKETACTION_ADDNEWPOSITION);

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShoppingList->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShoppingList' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShoppingList->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShoppingList = DE_EPAGES::ShoppingList::API::Object::ShoppingList->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShoppingList');
#               my $ShoppingList = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShoppingList' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShoppingList | new object | object
# hook         OBJ_InsertShoppingList | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShoppingListID - shopping list id     - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShoppingList', {'Object' => $self, 'ShoppingListID' => $self->id });
    delete $hInfo->{$_} foreach qw ( User Shop ChangeDate ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShoppingList->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShoppingList' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShoppingList | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShoppingListID - shopping list id     - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShoppingList', {'Object' => $self, 'ShoppingListID' => $self->id });
    LoadClassByAlias('ShoppingListItem')->deleteObjects('ByShoppingListID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShoppingList->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShoppingList' and 'OBJ_AfterUpdateShoppingList'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShoppingList | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShoppingListID - shopping list id     - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShoppingList | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShoppingListID - shopping list id     - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShoppingList', {'Object'=>$self, 'Values'=> $hValues, 'ShoppingListID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShoppingList',  {'Object'=>$self, 'Values'=> $hValues, 'ShoppingListID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ShoppingList::API::Object::ShoppingList->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'ChangeDate' => $hInfo->{'ChangeDate'} // DateTime->now,
    };
}

#========================================================================================
# function     addItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShoppingListItem = $ShoppingList->addItem($Product, $hInfo);
#----------------------------------------------------------------------------------------
# description  Add product to shopping list.
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# input        $hInfo | item info, keys :
#               <ul>
#                   <li>Quantity - product quantity (default: 1)</li>
#               </ul> | ref.hash
# return       $ShoppingListItem | list entry (undef if product not visible) | object
#========================================================================================
sub addItem {
    my $self = shift;
    my ($Product, $hInfo) = @_;

    return undef unless $Product->get('IsVisible');
    Error('NoMasterProductInShoppingList', { Path => $Product->pathString }) if $Product->get('IsMaster');

    $hInfo->{'Quantity'} ||= 1; # default quantity
    my $AddProductStyle = $hInfo->{'AddProductStyle'};

    # get items which are equal to product
    my $Item = $self->getItemByProduct($Product, $hInfo);

    if (not defined $Item or $AddProductStyle == BASKETACTION_ADDNEWPOSITION) {
        return $self->insertChild('ShoppingListItem', {
                'Alias'        => $self->nextAlias($Product->alias . '%s' ) ,
                'Product'      => $Product,
                'ShoppingList' => $self,
                'Quantity'     => $Product->correctQuantity($hInfo->{'Quantity'}),
        });
    }
    if ($AddProductStyle == BASKETACTION_DONTCHANGEQUANTITY) {
        delete $hInfo->{'Quantity'};
    } elsif ($AddProductStyle == BASKETACTION_REPLACEQUANTITY) {
        #nothing to do
    } elsif ($AddProductStyle == BASKETACTION_ADDQUANTITY) {
        $hInfo->{'Quantity'} = $Product->correctQuantity($hInfo->{'Quantity'} + $Item->get('Quantity'));
    }
    $Item->set($hInfo);
    return $Item;
}

#========================================================================================
# function     existsItemByProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ItemExists = $ShoppingList->existsItemByProduct($Product);
#----------------------------------------------------------------------------------------
# description  Test if a shopping list item exists by the given product
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# return       $ItemExists | true if item by product exists | boolean
#========================================================================================
sub existsItemByProduct {
    my $self = shift;
    my ($Product) = @_;

    return 1 if defined $self->getItemByProduct($Product);
    return 0;
}

#========================================================================================
# function     getItemByProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Item = $ShoppingList->getItemByProduct($Product);
#----------------------------------------------------------------------------------------
# description  Returns a shopping list item exists by the given product
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# return       $ShoppingListItem | list entry | object
#========================================================================================
sub getItemByProduct {
    my $self = shift;
    my ($Product, $hInfo) = @_;

    # get items which are equal to product
    my $Items = $self->get('Items');

    if (defined $hInfo->{'AddProductStyle'} && $hInfo->{'AddProductStyle'} == BASKETACTION_ADDNEWPOSITION) {
        return;
    }

    my ($Item) = grep { defined $_->get('Product') and $_->get('Product')->id == $Product->id} @$Items;

    return $Item;
}

1;
