#========================================================================================
# package      DE_EPAGES::ShoppingList::API::ShoppingList
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the interface for ShoppingList.
#========================================================================================
package DE_EPAGES::ShoppingList::API::ShoppingList;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    AddProductToShoppingList
    GetUnityShoppingList
);

use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestInteger TestString );
use DE_EPAGES::Order::API::Basket qw ( GetExistingBasket );
use DE_EPAGES::ShoppingList::API::Constants qw (UNITY_SHOPPINGLIST_ALIAS);
use DE_EPAGES::Shop::API::Constants qw (BASKETACTION_ADDNEWPOSITION);

#========================================================================================
# function     AddProductToShoppingList
#----------------------------------------------------------------------------------------
# syntax       AddProductToShoppingList($ShoppingList, $Product, $Shop, $hParams);
#----------------------------------------------------------------------------------------
# description  Adds product to the given shopping list.
#----------------------------------------------------------------------------------------
# input        $ShoppingList| shopping list | object
# input        $Product| product | object
# input        $Shop| shop | object
# input        $hParams| optional parameters
#                   <ul>
#                       <li>Quantity          - quantity default:1 - integer
#                       <li>LineItemGUID      - line item GUID when added via basket - string
#                       <li>User              - User object when added via login or registration - object
#                   </ul>
#               | ref.hash
#========================================================================================
sub AddProductToShoppingList {
    my ( $ShoppingList, $Product, $Shop, $hParams ) = @_;

    my $Quantity = $hParams->{'Quantity'};
    my $CustomizedText = $hParams->{'CustomizedTextUnity'};
    my $LineItemGUID = $hParams->{'LineItemGUID'};
    my $User = $hParams->{'User'};

    TestObject('ShoppingList',$ShoppingList,'DE_EPAGES::ShoppingList::API::Object::ShoppingList');
    TestObject('Product', $Product,'DE_EPAGES::Product::API::Object::Product');
    TestObject('Shop', $Shop,'DE_EPAGES::Shop::API::Object::Shop');
    TestInteger('Quantity', $Quantity) if $Quantity;
    TestString('LineItemGUID', $LineItemGUID) if $LineItemGUID;
    TestObject('User', $User,'DE_EPAGES::Permission::API::Object::User') if defined $User;

    # The Quantity is just set in case the related feature is set and a quantity value is given
    $Quantity = 1 unless (defined $Quantity && $Shop->featureMaxValue('UnityWishListShowQuantity'));
    my $AddProductStyle = defined  $CustomizedText ? BASKETACTION_ADDNEWPOSITION : $Shop->get('AddProductStyle');

    # get the existing or new ShoppingListItem
    my $ShoppingListItem = $ShoppingList->getItemByProduct($Product, {'AddProductStyle' => $AddProductStyle}) // $ShoppingList->addItem($Product, {'Quantity' => $Quantity,
                                                                                                                                                   'CustomizedTextUnity' => $CustomizedText,
                                                                                                                                                   'AddProductStyle' => $AddProductStyle});
    $ShoppingListItem->set({'CustomizedTextUnity' => $CustomizedText}) if defined($CustomizedText);

    if($LineItemGUID){
        # set the given quantity (do not increase the quantity)
        $ShoppingListItem->set({'Quantity' => $Quantity});

        if (defined $User) {
            # delete the line item from the basket,
            # if the item was added from the basket
            my $Basket = GetExistingBasket($User, $Shop);
            if (defined $Basket) {
                my $LineItemContainer = $Basket->get('LineItemContainer');
                my $LineItemAlias = 'LineItemProduct_' . $LineItemGUID;
                if (defined $LineItemContainer && $LineItemContainer->existsChild($LineItemAlias)) {
                    my $LineItem = $LineItemContainer->child($LineItemAlias);
                    $LineItem->delete();
                    $LineItemContainer->recalculate();
                }
            }
        }
    }
}

#========================================================================================
# function     GetUnityShoppingList
#----------------------------------------------------------------------------------------
# syntax       GetUnityShoppingList($User, $DoNotCreate);
#----------------------------------------------------------------------------------------
# description  Get the shopping list for an user
#               The shopping list is automatically created unless $DoNotCreate is given
#----------------------------------------------------------------------------------------
# input        $User| user | object
# input        $DoNotCreate (optional) | do not create the shopping list automatically | boolean
#----------------------------------------------------------------------------------------
# return       $ShoppingList | shopping list object | object
#========================================================================================
sub GetUnityShoppingList {
    my ( $User, $DoNotCreate ) = @_;

    TestObject('User',$User,'DE_EPAGES::Permission::API::Object::User');

    my $Shop   = $User->getSite();
    my $Folder = $User->folder('ShoppingLists');
    my $Alias  = UNITY_SHOPPINGLIST_ALIAS;
    my $ShoppingList;

    if ($Folder->existsChild($Alias) ){
        $ShoppingList = $Folder->child($Alias);
    } else {
        # create new ShoppingList
        $ShoppingList = $Folder->insertChild('ShoppingList',{
            'Alias' => $Alias,
            'User' =>  $User,
            'Shop' =>  $Shop,
        }) unless $DoNotCreate;
    }

    return $ShoppingList;
}

1;