#========================================================================================
# package      DE_EPAGES::ShoppingList::Hooks::Shop
# state        public
# description  interface for Shop
#========================================================================================
package DE_EPAGES::ShoppingList::Hooks::Shop;
use base qw( Exporter );

use strict;

use DE_EPAGES::Core::API::Url qw (ParseQueryString );
use DE_EPAGES::ShoppingList::API::Shop qw ( GetAddToWishlistScriptTagUrl ShoppingListsByShop );
use DE_EPAGES::ShoppingList::API::ShoppingList qw (AddProductToShoppingList GetUnityShoppingList);
use MIME::Base64 qw(decode_base64);

#========================================================================================
# function     OnDeleteShop
#----------------------------------------------------------------------------------------
# syntax       OnDeleteShop($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes all shopping lists of a shop when the shop is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Shop, key is
#               <ul>
#                   <li>Object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteShop {
    my ($hParams) = @_;
    $_->delete foreach @{ ShoppingListsByShop($hParams->{'Object'}) };
}

#========================================================================================
# function     OnRegisterInitialScriptTags
#----------------------------------------------------------------------------------------
# syntax       OnRegisterInitialScriptTags($hParams);
#----------------------------------------------------------------------------------------
# description  Adds the AddToWishlist script tag to the list of script tags that should
#                   be registered upon creation of a Unity shop
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameter
#               <ul>
#                   <li>Shop - the current Shop - object
#                   <li>ScriptTagUrls - list of scrip tag URLs - ref.array
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterInitialScriptTags {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Shop'};

    push @{$hParams->{'ScriptTagUrls'}}, GetAddToWishlistScriptTagUrl($Shop);
}

#========================================================================================
# function     OnAfterSaveRegistration
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterSaveRegistration( $hParams );
#----------------------------------------------------------------------------------------
# description  Adds the product from the base64 encoded parameter LastViewActionParameters
#               to te users shopping list
#               (Hook UI_AfterSaveRegistration)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet</li>
#                   <li>User - user object</li>
#                   <li>Customer - customer object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterSaveRegistration {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $User    = $hParams->{'User'};
    my $Shop    = $Servlet->object->getSite();

    return unless $Servlet->params('LastViewActionParameters');
    return unless defined $User;
    return if $User->get('IsAnonymous');

    my $Base64EncParameters = $Servlet->params('LastViewActionParameters');
    my $hValues             = ParseQueryString(decode_base64($Base64EncParameters),'utf-8');
    my $ProductGUID         = $hValues->{'ProductGUID'}->[0];
    my $Quantity            = $hValues->{'Quantity'}->[0];
    my $CustomizedText = $hValues->{'CustomizedText'}->[0];
    my $LineItemGUID        = $hValues->{'LineItemGUID'}->[0];

    return unless $ProductGUID;

    # add the product to the users shopping list
    my $ShoppingList = GetUnityShoppingList($User);
    my $Product      = $Shop->loadObjectByGuid($ProductGUID, 'Product');
    AddProductToShoppingList($ShoppingList, $Product, $Shop, {
        'Quantity'     => $Quantity,
        'LineItemGUID' => $LineItemGUID,
        'CustomizedTextUnity'    => $CustomizedText,
        'User'         => $Servlet->user,
    });
    return;
}

1;
