#========================================================================================
# package      DE_EPAGES::Zettle::API::Tools
# state        public
#========================================================================================
package DE_EPAGES::Zettle::API::Tools;
use base Exporter;

use strict;

use DateTime::Format::ISO8601;
use Time::HiRes qw ( time );

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( GetIPAddress Ip2Hex );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadRootObject );
use DE_EPAGES::Presentation::API::DereferrerToken qw ( GenerateDereferrerToken ValidateDereferrerToken );

our @EXPORT_OK = qw (
    FormatDateTime
    GenerateUUID
    GenerateCallbackToken
    GetProductError
    LoadProductByZettleUUID
    ParseDateTime
    ValidateCallbackToken

    LASTUPDATE_DATETIME_PATTERN
);

# Pattern used to format lastupdate datetime
use constant LASTUPDATE_DATETIME_PATTERN => '%d/%m/%Y %H:%M';

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'MultipleProductsByZettleUUID' => 'Multiple product results for ZettleUUID \'#ZettleUUID\'',
        'NoResults_ProductByZettleUUID' => 'No product found with ZettleUUID #ZettleUUID',
    }
}
#ERRORDEFINITION END


#========================================================================================
# function     GenerateUUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GUID = GenerateUUID();
#----------------------------------------------------------------------------------------
# description  Returns 36-character long strings containing UUIDv1 (i.e: 2091a200-1dd2-11b2-8000-080027b246c3).
#               See RFC: https://datatracker.ietf.org/doc/html/rfc4122#section-4
#               More info: https://www.uuidtools.com/uuid-versions-explained
#----------------------------------------------------------------------------------------
# return       $UUID | general unique identifier | string
#========================================================================================
sub GenerateUUID () {
    ## no critic qw(ProhibitMagicNumbers)
    my ($time_low, $time_mid, $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low)
        = (time, rand(256*256), rand(256*256), rand(256), rand(256));
    my $uuid = sprintf ( '%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%s%2.2x%2.2x', $time_low, $time_mid,
                     $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low,
                     lc(Ip2Hex(GetIPAddress(),8)), $$ % 256, rand(256) % 256 );
    # set UUID version (v1)
    substr($uuid, 14, 1, '1');
    # set UUID variant
    substr($uuid, 19, 1, '2');
    # force MAC multicast bit on per RFC
    my $multicast = substr($uuid, 25, 1);
    my $multicast_dec = hex($multicast);
    if(!($multicast_dec & 1)){
        $multicast = sprintf('%1.1x', $multicast_dec+1);
        substr($uuid, 25, 1, $multicast);
    }
    return $uuid;
    ## use critic
}

#========================================================================================
# function     GenerateCallbackToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Token = GenerateCallbackToken($Object);
#----------------------------------------------------------------------------------------
# description  Wrapper for GenerateDereferrerToken
#----------------------------------------------------------------------------------------
# input        $Object | any object that belongs to a shop | object
#----------------------------------------------------------------------------------------
# return       $CallbackToken | base 64 encoded token used to validate callbacks | String
#========================================================================================
sub GenerateCallbackToken {
    my ($Object) = @_;

    my $PayLoad = _GenerateValidationPayload($Object);

    return GenerateDereferrerToken($PayLoad);
}

#========================================================================================
# function     FormatDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DTString = FormatDateTime($DateTime);
# syntax       $DTString = FormatDateTime($DateTime, $Pattern);
#----------------------------------------------------------------------------------------
# description  Format a DateTime object into the string representation specified by $Pattern.
#----------------------------------------------------------------------------------------
# input        $DateTime | a DateTime object | string
# input        $Pattern | optional. Pattern to be used to generate string output
#                          (default: '%FT%T.%3N%z', i.e. 2021-05-31T13:48:49.978+0000)
#                          See https://metacpan.org/pod/DateTime#strftime-Patterns | string
#----------------------------------------------------------------------------------------
# return       $DTString | a string representation of given DateTime | string
#========================================================================================
sub FormatDateTime {
    my ($DateTime, $Pattern) = @_;

    return unless defined $DateTime;

    $Pattern //= '%FT%T.%3N%z';

    return $DateTime->strftime($Pattern);
}

#========================================================================================
# function     ParseDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = ParseDateTime($DTString);
# syntax       $DateTime = ParseDateTime($DTString, $TimeZone);
#----------------------------------------------------------------------------------------
# description  Parse the string representation of a date in ISO8601 format and return a DateTime object.
#----------------------------------------------------------------------------------------
# input        $DTString | a string representation of a datetime (sample: '2021-05-31T13:48:49.978+0000') | string
# input        $TimeZone | optional. Timezone that will be applied to resulting DateTime (default: 'Europe/Berlin') | string
#----------------------------------------------------------------------------------------
# return       $DateTime | a DateTime object | DateTime
#========================================================================================
sub ParseDateTime {
    my ($DTString, $TimeZone) = @_;

    return unless defined $DTString;

    $TimeZone //= 'Europe/Berlin';

    # Reference: https://metacpan.org/pod/DateTime::Format::ISO8601#SYNOPSIS
    my $DateTime = DateTime::Format::ISO8601->parse_datetime($DTString);
    # Reference: https://metacpan.org/pod/DateTime#$dt-%3Eset_time_zone($tz)
    $DateTime->set_time_zone($TimeZone);

    return $DateTime;
}

#========================================================================================
# function     ValidateCallbackToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Token = ValidateCallbackToken($Token, $Shop);
#----------------------------------------------------------------------------------------
# description  Verify $Token was previously generated by <function GenerateCallbackToken> using same Object.
#----------------------------------------------------------------------------------------
# input        $Token  | a token received as part of a callback request | string
# input        $Object | any object that belongs to a shop | object
#----------------------------------------------------------------------------------------
# return       $IsValid | TRUE if provided $Token is valid, false otherwise | String
#========================================================================================
sub ValidateCallbackToken {
    my ($Token, $Object) = @_;

    my $PayLoad = _GenerateValidationPayload($Object);

    return ValidateDereferrerToken($Token, $PayLoad);
}

sub _GenerateValidationPayload {
    my ($Object) = @_;

    my $ClientID = LoadRootObject()->get('Zettle_AppClientID');
    my $PayLoad = sprintf('%s&%s&%s', $Object->alias(), $Object->guid(), $ClientID);

    return $PayLoad;
}

#========================================================================================
# function     LoadProductByZettleUUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Product = LoadProductByZettleUUID($Shop, $ZettleUUID);
# example      $Product = LoadProductByZettleUUID($Shop, "b09e53f4-60e9-41a6-8b30-526b69c28b5d");
#----------------------------------------------------------------------------------------
# description  Returns the object by ZettleUUID.
#               Throws an error if no such objects exists.
#----------------------------------------------------------------------------------------
# throws       MultipleProductsByZettleUUID - if more than one product matches specified ZettleUUID
# throws       NoResults_ProductByZettleUUID - if no product matches specified ZettleUUID
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $ZettleUUID | general unique id of the object | string
#----------------------------------------------------------------------------------------
# return       $Product | a Product object  | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub LoadProductByZettleUUID {
    my ($Shop, $ZettleUUID) = @_;
    TestString( 'ZettleUUID', $ZettleUUID );

    #TODO: implement DAL for Product attributes and use index-based search for ZettleUUID attribute
    my $dbi = GetCurrentDBHandle();
    my $AttributeID = LoadClassByAlias('Product')->attribute('ZettleUUID')->id;
    my $Query = 'SELECT o.objectid FROM stringattribute sa
                    INNER JOIN object o ON (sa.attributeid=? AND sa.objectid=o.objectid AND o.siteid=?)
                 WHERE sa.value=?';
    my $ahRows = $dbi->execute($Query, [$AttributeID, $Shop->id, $ZettleUUID]);

    # Product UUIDs must be unique
    Error('MultipleProductsByZettleUUID', { 'ZettleUUID' => $ZettleUUID }) if (scalar @{$ahRows} > 1);
    
    Error("NoResults_ProductByZettleUUID", { 'ZettleUUID' => $ZettleUUID }) unless (scalar @{$ahRows});

    # A single result holds our product ID
    my $hRow = @{$ahRows}[0];
    my $ObjectID = @{$hRow}[0];
    return LoadObject($ObjectID);
}

#========================================================================================
# function     GetProductError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductError = GetProductError($Product, $Error)
#----------------------------------------------------------------------------------------
# description  Generate a ProductError object to handle errors that can happen during
#               export of products to Zettle.
#----------------------------------------------------------------------------------------
# input        $Product | a product object | DE_EPAGES::Product::API::Object::Product
# input        $Error | ePages error object | DE_EPAGES::Core::API::Error
#----------------------------------------------------------------------------------------
# return       $ProductError | a ProductError object | DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#========================================================================================
sub GetProductError {
    my ($Product, $Error) = @_;

    my $ProductError = DE_EPAGES::Zettle::API::WebService::Requirements::ProductError->new($Product, $Error);

    return $ProductError;
}

1;
