#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::ProductService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around Zettle Product and Image API
#               This service is based on the methods provided by Product API: https://products.izettle.com/swagger#/
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::ProductService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use HTTP::Status qw( HTTP_NO_CONTENT HTTP_NOT_FOUND );
use JSON;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Zettle::API::Tools qw ( GetProductError );
use DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO;
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw (
    GetAllProductsApiUrl
    GetBulkExportApiUrl
    GetBulkExportStatusApiUrl
    GetDeleteProductApiUrl
    GetDeleteProductListApiUrl
    GetProductApiUrl
    GetProductCountApiUrl
    GetProductUpdateApiUrl
);
use DE_EPAGES::Zettle::API::WebService::Requirements::ProductError;

# Maximum number of items to delete in a single bulk request
# See DE_EPAGES/Zettle/Documents/API/ZettleAPI-Extracts.md, Section #6.2.7.5. Delete a list of products: limitations
use constant BATCH_SIZE_DELETE => 45;

#ERRORDEFINITION
sub ErrorDefinition { return {
        'InitialExportError' => 'Initial export failed: #errorMessage',
        'ExportPricesError' => 'An error occured during prices export: #errorMessage'
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new ProductService object
#----------------------------------------------------------------------------------------
# input        $Environment | any Zettle environment object | DE_EPAGES::Object::API::Object:Object
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::ProductService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     exportProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hExportResult = $self->exportProducts($aProducts, $IncludePrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Export all products listed in $aProducts to Zettle.
#               This is an asynchronous process so export state shall be requested afterwards
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::getInitialExportState>.
#               API reference: https://products.izettle.com/swagger#/import/importLibraryV2
#----------------------------------------------------------------------------------------
# input        $aProducts | Products to be exported | ref.array.object
# input        $IncludePrices | Determines whether price information will be exported | boolean
#----------------------------------------------------------------------------------------
# return       $hExportResult | API response and list of filtered products, with following keys:
#               <ul>
#                   <li>response - zettle API response, with the following keys
#                       <ul>
#                           <li>uuid - export identifier (UUIDv1) - string
#                           <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" - string
#                           <li>items - number of items to be processed - integer
#                           <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                           <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                           <li>errorMessage - additional message when errors occur - string
#                       </ul>
#                   <li>failed - list of products that weren't exported due to errors - ref.array.DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#               </ul>| ref.hash
#----------------------------------------------------------------------------------------
# throws WebserviceError
#========================================================================================
sub exportProducts {
    my $self = shift;
    my ($aProducts, $IncludePrices, $hImageUploadResults) = @_;

    my $BulkExportUrl = GetBulkExportApiUrl();
    my $hBuildResult = $self->_buildBulkExportContent($aProducts, $IncludePrices, $hImageUploadResults);
    
    my $aProductDTO = $hBuildResult->{'products'};
    my $hResponse = $self->_sendExportRequest($BulkExportUrl, $aProductDTO);

    my $hExportResult = {
        'response' => $hResponse,
        'failed' => $hBuildResult->{'failed'}
    };

    return $hExportResult;
}

#========================================================================================
# function     updateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateProduct($Product, $ETag, $IncludePrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Updates a product entity using JSON merge patch. This means that only included fields will be changed.
#               API reference: https://products.izettle.com/swagger#/products/updateProduct
#----------------------------------------------------------------------------------------
# input        $Product | Product to be updated | DE_EPAGES::Product::API::Object::Product
# input        $ETag | ETag value as retrieved from GetProduct HttpResponse headers | string
# input        $IncludePrices | Determines whether price information will be exported | boolean
#----------------------------------------------------------------------------------------
# return       $Updated | TRUE if product was updated, false if product was not found | boolean
#----------------------------------------------------------------------------------------
# throws       WebserviceError
# throws       VariantsPerProductLimitExceeded | If number of subproducts exceed Zettle API limits.
#========================================================================================
sub updateProduct {
    my $self = shift;
    my ($Product, $ETag, $IncludePrices, $hImageUploadResults) = @_;

    # Compile shop product data and send it to Zettle
    my $ProductUUID = $Product->get('ZettleUUID');
    my $UpdateUrl = GetProductUpdateApiUrl($ProductUUID);
    my $hHeadersParams = {
        'Content-type' => 'application/json',
        'If-Match' => $ETag,
    };

    eval {
        my $hContent = $self->_buildUpdateContent($Product, $IncludePrices, $hImageUploadResults);
        $self->sendPutRequest($UpdateUrl, $hHeadersParams, $hContent);
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->vars()->{'Status'} == HTTP_NOT_FOUND){
            return 0;
        }
        Error($Error); #forward all other Errors
    }

    return 1;
}

#========================================================================================
# function     exportPrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->exportPrices($aProducts);
#----------------------------------------------------------------------------------------
# description  Export prices for the given products from the shop to Zettle.
#               This is an asynchronous process so export state shall be requested afterwards
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::getInitialExportState>.
#               API reference: https://products.izettle.com/swagger#/import/importLibraryV2
#----------------------------------------------------------------------------------------
# input        $aProducts | List of products | ref.array.object
#----------------------------------------------------------------------------------------
# return       $hExportResult | API response and list of filtered products, with following keys:
#               <ul>
#                   <li>response - zettle API response, with the following keys
#                       <ul>
#                           <li>uuid - export identifier (UUIDv1) - string
#                           <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS" - string
#                           <li>items - number of items to be processed - integer
#                           <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                           <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                           <li>errorMessage - additional message when errors occur - string
#                       </ul>
#                   <li>failed - list of products that weren't exported due to errors - ref.array.DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#               </ul>| ref.hash
#----------------------------------------------------------------------------------------
# throws WebserviceError
# throws ExportPricesError if API response indicates state is 'FINISHED_FAILED'
#========================================================================================
sub exportPrices {
    my $self = shift;
    my ($aProducts) = @_;

    my $BulkExportUrl = GetBulkExportApiUrl();
    my $hBuildResult = $self->_buildBulkExportPricesContent($aProducts);

    my $aProductDTO = $hBuildResult->{'products'};
    my $hResponse = $self->_sendExportRequest($BulkExportUrl, $aProductDTO);
    
    Error('ExportPricesError', $hResponse) if $hResponse->{state} eq 'FINISHED_FAILED';

    my $hExportResult = {
        'response' => $hResponse,
        'failed' => $hBuildResult->{'failed'}
    };

    return $hExportResult;
}

#========================================================================================
# function     getInitialExportState
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getInitialExportState();
#----------------------------------------------------------------------------------------
# description  Retrieve updated status from Zettle for bulk product export.
#----------------------------------------------------------------------------------------
# input        $ExportUUID | optional. Export UUID returned by Zettle | string
#----------------------------------------------------------------------------------------
# return       $State | state field from Zettle response: any of "IMPORTING", "FINISHED_SUCCESS" | string
#----------------------------------------------------------------------------------------
# throws       InitialExportError if returned state is 'FINISHED_FAILED' (Error is used to pass Zettle response to caller)
#========================================================================================
sub getInitialExportState {
    my $self = shift;
    my ($ExportUUID) = @_;

    # Query Zettle for current state of given bulk export
    my $ExportStatusURL = GetBulkExportStatusApiUrl($ExportUUID);
    
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };

    my $HttpResponse = $self->sendGetRequest($ExportStatusURL, $hHeadersParams);

    # {"uuid":"1aca6d65-e730-11eb-9cc8-b68971ccb386", "state":"FINISHED_SUCCESS", "items":10, "created":"2021-07-17T18:52:15.885+0000", "finished":"2021-07-17T18:52:15.927+0000", "errorMessage": null}
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    my $State = $hResponse->{state};
    Error('InitialExportError', $hResponse) if $State eq 'FINISHED_FAILED';

    return $State;
}

#========================================================================================
# function     getProductCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getProductCount();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/countAllProducts
#----------------------------------------------------------------------------------------
# return       $hResponse | Product count in a hash with the key 'productCount' | ref.hash
#========================================================================================
sub getProductCount {
    my $self = shift;

    my $Url = GetProductCountApiUrl();

    my $HttpResponse = $self->sendGetRequest($Url);
    
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

#========================================================================================
# function     getProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HttpResponse = $self->getProduct($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve the specifed product from Zettle.
#               Raw HttpResponse is returned since 'ETag' header is required in caller.
#               See: https://products.izettle.com/swagger#/products/getProduct
#----------------------------------------------------------------------------------------
# return       $HttpResponse | An HttpResponse object | HttpResponse
#========================================================================================
sub getProduct {
    my $self = shift;
    my ($ProductUUID) = @_;

    my $GetProductUrl = GetProductApiUrl($ProductUUID);

    my $HttpResponse = $self->sendGetRequest($GetProductUrl);
    
    # my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $HttpResponse;
}

#========================================================================================
# function     getAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductResponseDTO = $self->getAllProducts();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/getAllProductsInPos
#               See: https://products.izettle.com/swagger#/products/getAllProductsV2
#----------------------------------------------------------------------------------------
# return       $aProductResponseDTO | Array of ProductResponseDTO (see DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO) | ref.array.object
#========================================================================================
sub getAllProducts {
    my $self = shift;

    my $Url = GetAllProductsApiUrl();

    #TODO: include 'sort' parameter in request if passed as an argument in method call
    my $HttpResponse = $self->sendGetRequest($Url);
    
    #TODO: check why decodeResponseContent() returns undef when processing a considerable amount of products (more than 50 products)
    # my $hResponse = $self->decodeResponseContent($HttpResponse);    
    # print "hResponse=".Data::Dumper::Dumper($hResponse)."\n";
    
    my $DecodedContent = $HttpResponse->decoded_content();
    # print "DecodedResponse=".Data::Dumper::Dumper($DecodedResponse)."\n";
    my $hResponse = decode_json($DecodedContent);

    my $aProductResponseDTO;
    foreach my $ProductResponse (@{$hResponse}){
        my $ProductResponseDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO->parseResponse($ProductResponse);
        push(@{$aProductResponseDTO}, $ProductResponseDTO);
    }

    return $aProductResponseDTO;
}

#========================================================================================
# function     deleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Deleted = $self->deleteProduct($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Delete a single product from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $Deleted | TRUE if product was deleted, false if product was not found | boolean
#========================================================================================
sub deleteProduct {
    my $self = shift;
    my ($ProductUUID) = @_;

    my $Url = GetDeleteProductApiUrl($ProductUUID);

    eval {
        $self->sendDeleteRequest($Url);
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->vars()->{'Status'} == HTTP_NOT_FOUND){
            return 0;
        }
        Error($Error); #forward all other Errors
    }

    return 1;
}

#========================================================================================
# function     deleteProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aDeletedIDs = $self->deleteProducts($aProductUUIDs);
#----------------------------------------------------------------------------------------
# description  Delete a list of products from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProducts
#               See: https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/delete-products#delete-multiple-products-in-one-request
#----------------------------------------------------------------------------------------
# input        $aProductUUIDs | List of ZettleUUIDs of products to delete | ref.array.string
#----------------------------------------------------------------------------------------
# return       $hDeleteResult | List of deleted products and list of failed deletions, with following keys:
#               <ul>
#                   <li>deleted - list of ZettleUUIDs of products successfully deleted from Zettle - ref.array.string
#                   <li>failed - list of ZettleUUIDs of products that couldn't be deleted - ref.array.string
#               </ul>| ref.hash
#========================================================================================
sub deleteProducts {
    my $self = shift;
    my ($aProductUUIDs) = @_;

    my $Url = GetDeleteProductListApiUrl();

    my $hDeleteResult = {
        'deleted' => [],
        'failed' => []
    };
    my $ProductCount = scalar @{$aProductUUIDs};
    # Only 45 products may be safely deleted at once (in one request).
    # We group products in batches and send as many requests as necessary.
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=BATCH_SIZE_DELETE){
        my $EndIndex = ($StartIndex+BATCH_SIZE_DELETE >= $ProductCount) ? $ProductCount-1 : $StartIndex+BATCH_SIZE_DELETE-1;
        my @aBatchProductUuid = @{$aProductUUIDs}[$StartIndex..$EndIndex];

        my $hContent = {
            'uuid' => \@aBatchProductUuid
        };
        # Zettle wrongfully defined delete products API request with HTTP Parameter Pollution (HPP): multiple parameters with the same key (uuid) and different values.
        # In order to construct query parameters we use https://metacpan.org/pod/URI#$uri-%3Equery_form(-\%hash-) passing a reference to an array of strings.
        my $hOptions = {
            'content' => $hContent,
        };

        my $HttpResponse = $self->sendDeleteRequest($Url, $hOptions);
        
        # Compile the list of IDs for successful and failed deletions
        if ($HttpResponse->code == HTTP_NO_CONTENT) {
            push (@{$hDeleteResult->{'deleted'}}, @aBatchProductUuid);
        } else {
            push (@{$hDeleteResult->{'failed'}}, @aBatchProductUuid);
        }
    }

    return $hDeleteResult;
}


#========================================================================================


#========================================================================================
# function     _buildBulkContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hBuildResult = $self->_buildBulkContent($aProducts, $ExportPrices, $hImageUploadResults)
#----------------------------------------------------------------------------------------
# description  Processes given products and generates the ProductDTO expected by Zettle Product API.
#               If it's a price export, $ExportPrices and $hImageUploadResults must not be provided.
#               This is a wrapper function to be called by _buildBulkExportContent and _buildBulkExportPricesContent methods.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of products | ref.array.object
# input        $ExportPrices | Optional. Only required for product export, not for price-only export.
#                               TRUE if prices shall be included, FALSE otherwise | boolean
# input        $hImageUploadResults | Optional. Only required for product export, not for price-only export.
#                                      A map of ImageUploadResult objects indexed by product GUID.
#                                      Image lookupKeys for products to be exported will be searched for here.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#----------------------------------------------------------------------------------------
# return       $hBuildResult | a hash with the following keys:
#               <ul>
#                  <li>products - list of ProductDTO-based objects built (see <DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO)>) - ref.hash
#                  <li>failed - list of products whose converstion to DTO failed - ref.array.DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#               </ul> | ref.hash
#========================================================================================
sub _buildBulkContent {
    my $self = shift;
    my ($aProducts, $ExportPrices, $hImageUploadResults) = @_;
    
    my $hBuildResult = {
        'products' => [],
        'failed' => []
    };

    # Copy original array to avoid side effects on caller method
    my $aProductsCopy = [ @{$aProducts} ];

    my $Environment = $self->getEnvironment();
    my $Shop = $Environment->getSite();
    my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
    
    # Map of ProductDTOs indexed by product UUID used to merge VariationDTO data for subproducts with the same parent product
    my $hProducts = {};
    my @ZettleProducts = ();
    my $ProductDTO;
    my $hFilteredProducts = {};
    foreach my $Product ( @{$aProductsCopy} ) {
        eval {
            # If $ExportPrices is present we need to build a ProductDTO, otherwise it's a price-only export
            if (defined $ExportPrices) {
                $ProductDTO = $Factory->buildProductDTO($Product, $ExportPrices, $hImageUploadResults);
            } else {
                $ProductDTO = $Factory->buildProductPriceDTO($Product);
            }

            my $Uuid = $ProductDTO->uuid();
            if (exists $hProducts->{$Uuid}) {
                my $ExistingProductDTO = $hProducts->{$Uuid};
                _addUniqueVariationsToProductDTO($ExistingProductDTO,$ProductDTO->variations);
            } else {
                $hProducts->{$Uuid} = $ProductDTO;
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $ProductError = GetProductError($Product, $Error);
            push(@{$hBuildResult->{'failed'}}, $ProductError);
            $hFilteredProducts->{$ProductError->id} = 1;
        } else {
        }
    }

    #PATCH: bulk export would remove variations not present in update hence we need to include all variations
    foreach my $Product ( @{$aProductsCopy} ) {
        # Do not populate variations for products filtered in previous step
        next if defined($hFilteredProducts->{$Product->id});

        $Product = $Product->superProduct if defined $Product->superProduct;
        # Do not populate variations unless ZettleUUID is defined for parent
        my $ProductUuid = $Product->get('ZettleUUID');
        next unless defined $ProductUuid;
        
        my $aSubProducts = $Product->get('SubProducts');
        if (scalar @{$aSubProducts} > 0) {
            foreach my $SubProduct (@$aSubProducts) {
                $ProductDTO = $hProducts->{$ProductUuid};
                next unless defined $ProductDTO;
                my $VariationUuid = $SubProduct->get('ZettleUUID');
                next unless defined $VariationUuid;
                eval {
                    my $aProductVariationDTOs = $ProductDTO->variations();
                    unless(grep { $_->uuid eq $VariationUuid} @{$aProductVariationDTOs}){
                        push(@{$aProductVariationDTOs}, $Factory->buildPatchVariationDTO($SubProduct));
                    }
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    my $ProductError = GetProductError($Product, $Error);
                    push(@{$hBuildResult->{'failed'}}, $ProductError);
                }

            }
        }
    }

    foreach my $UuidKey (keys %{$hProducts}){
        $ProductDTO = $hProducts->{$UuidKey};
        push(@ZettleProducts, $ProductDTO->toHash());
    }

    $hBuildResult->{'products'} = \@ZettleProducts;

    return $hBuildResult;
}

#========================================================================================
# function     _buildBulkExportContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hBuildResult = $self->_buildBulkExportContent($aProducts, $ExportPrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Processes given products and generates the ProductDTO expected by Zettle Product API.
#               It will include any previously exported image available in $hImageUploadResults.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of products | ref.array.object
# input        $ExportPrices | TRUE if prices shall be included, FALSE otherwise | boolean
# input        $hImageUploadResults | a map of ImageUploadResult objects indexed by product GUID
#                                      Image lookupKeys for products to be exported will be searched here.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#----------------------------------------------------------------------------------------
# return       $hBuildResult | a hash with the following keys:
#               <ul>
#                  <li>products - list of ProductDTO-based objects built (see <DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO)>) - ref.hash
#                  <li>failed - list of products whose converstion to DTO failed - ref.array.DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#               </ul> | ref.hash
#========================================================================================
sub _buildBulkExportContent {
    my $self = shift;
    my ($aProducts, $ExportPrices, $hImageUploadResults) = @_;
    
    my $hBuildResult = $self->_buildBulkContent($aProducts, $ExportPrices, $hImageUploadResults);

    return $hBuildResult;
}

#========================================================================================
# function     _addUniqueVariationsToProductDTO
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hContent = _addUniqueVariationsToProductDTO($ExistingProductDTO,$ahNewVariations);
#----------------------------------------------------------------------------------------
# description  Add new variations to ProductDTO variations if these variation do not exist
#               in its variation array element
#----------------------------------------------------------------------------------------
# input        $ExistingProductDTO | hash that has a 'variations' key  | ref.hash
# input        $ahNewVariations | The new variation hashes we want to add | ref.hash.array
#========================================================================================
sub _addUniqueVariationsToProductDTO {
    my ($ExistingProductDTO,$ahNewVariations) = @_;
    
    foreach my $hNewVariation (@$ahNewVariations){
        my $newVariationAlreadyExists = 0;
        foreach my $hCurrentVariation (@{$ExistingProductDTO->variations}){
            if($hNewVariation->{uuid} eq $hCurrentVariation->{uuid}){
                $newVariationAlreadyExists = 1;
                last;
            }
        }
        if(!$newVariationAlreadyExists){
            push(@{$ExistingProductDTO->variations}, $hNewVariation);
        }
    }
}

#========================================================================================
# function     _buildUpdateContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hContent = $self->_buildUpdateContent($Product, $ExportPrices, $hImageUploadResults);
#----------------------------------------------------------------------------------------
# description  Process given product and generate the contents expected by Zettle Product API.
#               It will include any previously exported image available in $hImageUploadResults.
#----------------------------------------------------------------------------------------
# input        $Product | a product object | DE_EPAGES::Product::API::Object::Product
# input        $ExportPrices | TRUE if prices shall be included, FALSE otherwise | boolean
# input        $hImageUploadResults | a map of ImageUploadResult objects indexed by product GUID
#                                      Image lookupKeys for products to be exported will be searched here.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#----------------------------------------------------------------------------------------
# return       $hContent | export content as expected by Product API (based on DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO) | ref.hash
#========================================================================================
sub _buildUpdateContent {
    my $self = shift;
    my ($Product, $ExportPrices, $hImageUploadResults) = @_;
    
    my $Environment = $self->getEnvironment();
    my $Shop = $Environment->getSite();
    my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
    my $ProductDTO = $Factory->buildProductDTO($Product, $ExportPrices, $hImageUploadResults);

    return $ProductDTO->toHash();
}

#========================================================================================
# function     _buildBulkExportPricesContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hBuildResult = $self->_buildBulkExportPricesContent($aProducts);
#----------------------------------------------------------------------------------------
# description  Process given products/subproducts and generate the contents expected by
#               Zettle Product API in order to only update prices.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of products/subproducts | ref.array.object
#----------------------------------------------------------------------------------------
# return       $hBuildResult | a hash with the following keys:
#               <ul>
#                  <li>products - list of ProductDTO-based objects built (see <DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO)>) - ref.hash
#                  <li>failed - list of products whose converstion to DTO failed - ref.array.DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#               </ul> | ref.hash
#========================================================================================
sub _buildBulkExportPricesContent {
    my $self = shift;
    my ($aProducts) = @_;

    my $hBuildResult = $self->_buildBulkContent($aProducts);

    return $hBuildResult;
}

#========================================================================================
# function     _sendExportRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->_sendExportRequest($ExportUrl, $aProductDTO, $hHeadersParams)
#----------------------------------------------------------------------------------------
# description  Send an export request to Zettle and retrieve result.
#----------------------------------------------------------------------------------------
# input        $ExportUrl | Target URL for POST request | string
# input        $aProductDTO | List of product DTOs ready for export | ref.array.DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO
# input        $hHeadersParams | Optional. Additional param (Default: {'Content-type' => 'application/json'}).
#                                 Override default 'Content-type' by adding your own type to hash | ref.hash
#----------------------------------------------------------------------------------------
# return       $hResponse | API response with following keys:
#               <ul>
#                   <li>uuid - export identifier (UUIDv1) - string
#                   <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" - string
#                   <li>items - number of items to be processed - integer
#                   <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                   <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                   <li>errorMessage - additional message when errors occur - string
#               </ul>| ref.hash
#----------------------------------------------------------------------------------------
# throws WebserviceError
#========================================================================================
sub _sendExportRequest {
    my $self = shift;
    my ($ExportUrl, $aProductDTO, $hHeadersParams) = @_;

    if (defined $hHeadersParams) {
        $hHeadersParams->{'Content-Type'} = 'application/json' unless exists $hHeadersParams->{'Content-Type'};
    } else {
        $hHeadersParams = {
            'Content-type' => 'application/json'
        };
    }

    # Construct Zettle payload from properly built DTOs
    my $hContent = { 'products' =>  $aProductDTO };

    # Send request to Zettle and decode response
    my $HttpResponse = $self->sendPostRequest($ExportUrl, $hHeadersParams, $hContent);
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

1;
