#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::Requirements::Product
# state        public
#----------------------------------------------------------------------------------------
# description  class to ensure that product operations are compliant with Zettle API requirements
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::Requirements::Product;
use base Exporter;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Zettle::API::Constants qw ( API_MAX_VARIATIONS_PER_PRODUCT );
use DE_EPAGES::Zettle::API::Tools qw ( GetProductError );
use DE_EPAGES::Zettle::API::WebService::Requirements::ProductError;

our @EXPORT_OK = qw (
    FilterExportableProducts
    TestRequirements
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'VariantsPerProductLimitExceeded' => 'Number of subproducts (#SubProductCount) exceed Zettle API limit (#MaxVariantsPerProduct) for product ID=#ProductId',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     TestRequirements
# state        public
#----------------------------------------------------------------------------------------
# syntax       TestRequirements($Product);
#----------------------------------------------------------------------------------------
# description  Test if given $Product meets criteria in order to be exported to Zettle.
#               It will throw an error if product doesn't meet requirements:
#               <ul>
#                   <li>Number of subproducts is below 100.
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Product | a product object | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# throws       VariantsPerProductLimitExceeded | If number of subproducts (or siblings) exceed Zettle API limits.
#========================================================================================
sub TestRequirements {
    my ($Product) = @_;

    # Test number of subproducts meet Zettle requirements
    _TestVariationsPerProduct($Product);
}

#========================================================================================
# function     FilterExportableProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($aExportableProducts, $hFilteredProducts) = FilterExportableProducts($aProducts);
#----------------------------------------------------------------------------------------
# description  Retrieve products that match requirements to be exported to Zettle.
#               See <function TestRequirements>.
#----------------------------------------------------------------------------------------
# input        $aProducts | a list of product objects | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       ($aExportableProducts, $hFilteredProducts) | two elements, first item is a list
#               of exportable products and second item is a map formed by ProductError objects
#               indexed by product id (see DE_EPAGES::Zettle::API::WebService::Requirements::ProductError) | array
#========================================================================================
sub FilterExportableProducts {
    my ($aProducts) = @_;
    
    my $aExportableProducts = [];
    my $hFilteredProducts = {};
    foreach my $Product (@{$aProducts}) {
        eval {
            TestRequirements($Product);
        };
        if (ExistsError()) {
            my $ProductError = GetProductError($Product, GetError());
            $hFilteredProducts->{$ProductError->id} = $ProductError;
        } else {
            push (@{$aExportableProducts}, $Product);
        }
    }

    return ($aExportableProducts, $hFilteredProducts);
}


#========================================================================================


#========================================================================================
# function     _TestVariationsPerProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TestVariationsPerProduct($Product)
#----------------------------------------------------------------------------------------
# description  Test if given $Product meets criteria in order to be exported to Zettle
#               in terms of subproduct count. Zettle API imposes a limit on the number of subproducts.
#               A master product will not be exported if the number of subproducts exceeds Zettle API limits.
#               A variation product will not be exported if the number of subproducts
#               of the parent exceeds Zettle API limits.
#               See DE_EPAGES/Zettle/Documents/API/ZettleAPI-MaxVariationsPerProduct.md
#----------------------------------------------------------------------------------------
# input        $Product | a product (or subproduct) object | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# throws       VariantsPerProductLimitExceeded | If number of subproducts (or siblings) exceed Zettle API limits.
#========================================================================================
sub _TestVariationsPerProduct {
    my ($Product) = @_;

    my $IsVariation = defined $Product->superProduct();
    # If dealing with master product, we consider the number of subproducts
    # If dealing with subproduct, we consider the number of siblings (number of subproducts of parent product)
    my $SuperProduct = $IsVariation ? $Product->superProduct() : $Product;
    
    my $SubProductCount = $SuperProduct->countSubProducts();

    # Ensure number of variations is compliant with API limits
    # - SuperProduct fails check when number of subproducts is over the limit.
    # - SubProduct fails check when number of siblings is over the limit AND subproduct does not have ZettleUUID:
    #   this condition allows previously exported variations to be updated in Zettle.
    if ($SubProductCount > API_MAX_VARIATIONS_PER_PRODUCT &&
        (!$IsVariation || !defined $Product->get('ZettleUUID'))) {
        Error('VariantsPerProductLimitExceeded',
                {
                    'ProductId' => $Product->id,
                    'IsVariation' => $IsVariation,
                    'SubProductCount' => $SubProductCount,
                    'MaxVariantsPerProduct' => API_MAX_VARIATIONS_PER_PRODUCT
                });
    }
}

1;