#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
# state        public
#----------------------------------------------------------------------------------------
# description  class to store information of errors occurred during product export to Zettle.
#----------------------------------------------------------------------------------------
# notes        The information is structured in the following way:
#               <ul>
#                   <li> id - ID of affected product - int
#                   <li> zettleuuid - UUID generated for Zettle - string
#                   <li> code - id of generated error - string
#                   <li> reason - reason for generated error or for filtering product - string
#                   <li> timestamp - time when error was registered - DateTime
#               </ul>
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::Requirements::ProductError;

use strict;

use JSON qw ( to_json );

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter  qw( TestObject );
use DE_EPAGES::Zettle::API::Tools qw ( FormatDateTime );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::Requirements::ProductError->new($Product, $Error);
# syntax       DE_EPAGES::Zettle::API::WebService::Requirements::ProductError->new($Product, $Error, $DateTime);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Product | a product object | DE_EPAGES::Product::API::Object::Product
# input        $Error | ePages error object | DE_EPAGES::Core::API::Error
# input        $DateTime | optional. Time when error was registered (default: current datetime) | Datetime
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::Requirements::ProductError
#========================================================================================
sub new {
    my $class = shift;
    my ($Product, $Error, $DateTime) = @_;
    $DateTime //= GetCurrentDateTime();

    TestObject('Product', $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestObject('Error', $Error, 'DE_EPAGES::Core::API::Error');

    my $Message = $Error->message;
    # Remove code reference from message
    $Message =~ s/ at $ENV{EPAGES_CARTRIDGES}.*//;
    my $ZettleUUID = $Product->get('ZettleUUID');
    my $self = bless {
        'id'        => $Product->id,
        'zettleuuid'=> $ZettleUUID,
        'code'      => $Error->code,
        'reason'    => $Message,
        'timestamp' => $DateTime
    }, $class;

    return $self;
}

sub id {
    my $self = shift;
    return $self->{'id'};
}

sub zettleuuid {
    my $self = shift;
    return $self->{'zettleuuid'};
}

sub code {
    my $self = shift;
    return $self->{'code'};
}

sub reason {
    my $self = shift;
    return $self->{'reason'};
}

sub timestamp {
    my $self = shift;
    return $self->{'timestamp'};
}

# This method is invoked internally when <toJson> method is called.
sub TO_JSON {
    my $self = shift;
    my $hDatetimeAttributes = {};
    my $Timestamp = $self->timestamp();
    $hDatetimeAttributes->{'timestamp'} = FormatDateTime($Timestamp) if defined $Timestamp;
    my $Json = { %{ $self }, %{ $hDatetimeAttributes } };
    return $Json;
}

#========================================================================================
# function     toJson
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductErrorJSON = $self->toJson();
#----------------------------------------------------------------------------------------
# description  Generate a JSON representation of current object.
#----------------------------------------------------------------------------------------
# return       $ProductErrorJSON | a JSON string | string
#========================================================================================
sub toJson {
    my $self = shift;

    return to_json($self, { 'pretty' => 1, 'allow_blessed' => 1, 'convert_blessed' => 1 });
}

1;
