#========================================================================================
# package      DE_EPAGES::Zettle::Hooks::Product
# state        public
# description  export stock level updates to Zettle
#========================================================================================
package DE_EPAGES::Zettle::Hooks::Product;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime Compare );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( IsCalledInBatchMode );
use DE_EPAGES::Zettle::API::Constants qw (
    API_IMAGE_FORMAT
    API_MAX_VARIATIONS_PER_PRODUCT
    ZETTLE_BASIC_ATTRIBUTES
    ZETTLE_ATTRIBUTES_PROFILE
    ZETTLE_IMAGE_PROFILE
    ZETTLE_PRICE_PROFILE
    ZETTLE_OBJECT_ALIAS
);
use DE_EPAGES::Zettle::API::WebService::InventoryService;
use DE_EPAGES::Zettle::API::ZettleService;
use DE_EPAGES::Zettle::DAL::Zettle qw( DAL_SetZettleDeleted );


# Contexts to check in order to ensure we don't update Zettle profile from within BatchDelete
# DeleteObjects context is not considered since it's used when deleting variations and would break expected behaviour
use constant BATCH_DELETE_CONTEXTS => qw (
    DeleteProducts
    DeleteAllProducts
);

# Profiles that need to be updated when product is compliant with API limits after subproduct deletion
use constant ZETTLE_PROFILES_FOR_API_LIMITS => [
    ZETTLE_ATTRIBUTES_PROFILE,
    ZETTLE_PRICE_PROFILE
];

#========================================================================================
# function     OnBeforeUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  Export stock level updates to Zettle and update Zettle-related LastUpdateProfiles if necessary.
#               (Hook OBJ_BeforeUpdateProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $Shop = $Product->getSite();

    return unless $Shop->featureMaxValue('Zettle');

    return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

    # skip if merchant disconnected from Zettle
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    return if $Environment->requiresCleanup();

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    return unless $ZettleService->isInitialExportComplete();

    _ExportStock($hParams);

    _UpdateProfiles($hParams);

    return;
}

#========================================================================================
# function     OnDeleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  OnDeleteProduct is triggered when a product is deleted and we have to:
#               <ul>
#                   <li>mark superproduct as modified if deleted product is a variation
#                       and number of subproducts would equal API_MAX_VARIATIONS_PER_PRODUCT.
#                   <li>disable Zettle inventory tracking for the product deleted, if required.
#                   <li>store ZettleUUID in zettledelete table.
#               </ul>
#               (Hook OBJ_DeleteProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - product object       - object
#                 <li>ProductID     - product identifier   - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};

    my $Shop = $Product->getSite();

    return unless $Shop->featureMaxValue('Zettle');

    return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

    # skip if merchant disconnected from Zettle
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    return if $Environment->requiresCleanup();

    my $SuperProduct = $Product->superProduct();
    _UpdateZettleProfilesOnVariationsPerProduct($SuperProduct) if defined $SuperProduct;

    my $ZettleUUID = $Product->get('ZettleUUID');
    return unless defined $ZettleUUID;

    # Disable inventory tracking for superproducts whose ZettleTracking is enabled
    my $TrackingEnabled = $Product->get('ZettleTrackingEnabled');
    if($TrackingEnabled && ! defined $SuperProduct){
        eval{
            my $InventoryService = DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
            $InventoryService->disableProductTracking($ZettleUUID);
        };
        if(ExistsError()){
            my $Error = GetError();
            LogError($Error);
        }
    }

    # Register ZettleUUID in zettledelete table:
    # this data is used during scheduled or manual product export to send deletions to Zettle
    eval{
        DAL_SetZettleDeleted({
            GUID => $Product->guid,
            ClassID => LoadClassByAlias('Product')->id,
            SiteID => $Shop->id,
            PathFromSite => $Product->get('RelativePath'),
            ZettleUUID => $ZettleUUID,
        });
    };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
    }
}


#========================================================================================


#========================================================================================
# function     _ExportStock
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ExportStock( $hParams );
#----------------------------------------------------------------------------------------
# description  Export stock level updates to Zettle
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub _ExportStock {
    my ($hParams) = @_;

    # only react if stocklevel changed.
    return unless defined $hParams->{'Values'};
    return unless defined $hParams->{'Values'}->{'StockLevel'};

    # Prevent incoming stock updates (Zettle webhooks) to be sent back to Zettle
    if(defined $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'} && $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'} == 1){
        delete $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'};
        return;
    }

    my $Product = $hParams->{'Object'};
    my $hNewValues = $hParams->{'Values'};
    my $NewStockLevel = $hNewValues->{'StockLevel'};

    my $Shop = $Product->getSite();
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    my $InventoryService = DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
    my $OldStockLevel = $Product->get('StockLevel') // 0;

    my $SuperProduct = $Product->superProduct;
    my $ProductUuid = (defined $SuperProduct) ? $SuperProduct->get('ZettleUUID') : $Product->get('ZettleUUID');
    my $VariantUuid = $Product->get('ZettleUUID');

    # Avoid sending stock updates when a product is cloned (apparently set({'StockLevel' ...}) is called and triggers this call)
    # Also prevent sending updates for variations that go over Zettle API limits (which don't have a ZettleUuid value)
    return unless defined $ProductUuid && defined $VariantUuid;
    
    eval {
        my $BalanceChange = $NewStockLevel-$OldStockLevel;
        if ($BalanceChange != 0) {
            #TODO: process ProductBalanceDTO returned and log a warning if Zettle balance doesn't match $NewStockLevel
            $InventoryService->updateProductBalance($ProductUuid, $VariantUuid, $BalanceChange);
            #TODO: process WS response and do not update stockSync if there was an error
            $Environment->lastStockSync(GetCurrentDateTime());
        }
    };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
    }

    return;
}

#========================================================================================
# function     _UpdateProfiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UpdateProfiles( $hParams );
#----------------------------------------------------------------------------------------
# description  Update Zettle LastUpdateProfiles if any attribute of interest was modified.
#               Register changes with regards to two different profiles:
#               <ul>
#                   <li>ZettleAttributes - changes in basic product attributes of interest to Zettle export
#                   <li>ImageLarge - changes in product image
#               </ul>
#               A separate ImageLarge profile is used since images are handled and uploaded in different API endpoints (ImageService vs ProductService)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub _UpdateProfiles {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $Shop = $Product->getSite();
    my $LanguageID = $Shop->get('LanguageID');
    my $hNewValues = $hParams->{'Values'};

    # Register changes in basic product attributes
    my @ZettleAttributes = ( ZETTLE_BASIC_ATTRIBUTES );
    if (my @UpdatedAttributes = grep { exists $hNewValues->{$_} } @ZettleAttributes) {
        my $hOldValues = $Product->get(\@UpdatedAttributes, $LanguageID);
        if ( grep { Compare('String', $hNewValues->{$_}, $hOldValues->{$_}) != 0} @UpdatedAttributes) {
            my $ZettleAttributesProfile = LoadClassByAlias('Product')->lastUpdateProfile(ZETTLE_ATTRIBUTES_PROFILE);
            $Product->setUpdated( $ZettleAttributesProfile );
        }
    }

    # Register changes in product image (ImageLarge)
    my $ImageAttribute = API_IMAGE_FORMAT;
    if (exists $hNewValues->{$ImageAttribute}) {
        my $imageOldValue = $Product->get($ImageAttribute);
        if (Compare('String', $hNewValues->{$ImageAttribute}, $imageOldValue) != 0) {
            my $ZettleImageProfile = LoadClassByAlias('Product')->lastUpdateProfile(ZETTLE_IMAGE_PROFILE);
            $Product->setUpdated( $ZettleImageProfile );
        }
    }
}

sub _IsBatchDeleteCall {
    my $BatchMode = 0;

    foreach my $Context (BATCH_DELETE_CONTEXTS) {
        if (IsCalledInBatchMode($Context)) {
            $BatchMode = 1;
            last;
        }
    }

    return $BatchMode;
}

#========================================================================================
# function     _UpdateZettleProfilesOnVariationsPerProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UpdateZettleProfilesOnVariationsPerProduct($SuperProduct);
#----------------------------------------------------------------------------------------
# description  Check whether deletion of a variation would make superproduct comply with Zettle API limits.
#               If number of variations for superproduct would equal API_MAX_VARIATIONS_PER_PRODUCT,
#               after deleting variation, then we mark superproduct as modified for Zettle profiles.
#               This allows us to export products to Zettle that were previously ignored for having
#               more than API_MAX_VARIATIONS_PER_PRODUCT subproducts.
#----------------------------------------------------------------------------------------
# input        $SuperProduct | a master product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub _UpdateZettleProfilesOnVariationsPerProduct {
    my ($SuperProduct) = @_;

    my $SubProductCount = $SuperProduct->countSubProducts();
    if ($SubProductCount-1 == API_MAX_VARIATIONS_PER_PRODUCT
            # Avoid setUpdated() to be called within BatchDelete since it would raise a FK constraint error (fk_lastupdate_objectid)
            && !_IsBatchDeleteCall()) {
        eval {
            foreach my $Profile (@{+ZETTLE_PROFILES_FOR_API_LIMITS}) {
                my $LastUpdateProfile = LoadClassByAlias('Product')->lastUpdateProfile($Profile);
                # Mark parent as modified so that it's collected in the following export to Zettle
                $SuperProduct->setUpdated($LastUpdateProfile);
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
        }
    }
}

1;
